#!/usr/bin/env python

"""
subfig package

TO-DO
- Options handling
- Only works with figure environment
- Lists of floats

"""

from plasTeX import Command, Environment
from plasTeX.Base.LaTeX.Floats import Float

def ProcessOptions(options, document):
    context = document.context
    context.newcounter('subfloat', resetby='figure', format='${subfloat.alph}')

class newsubfloat(Command):
    args = '[ options:dict ] name:str'
    def invoke(self, tex):
        Command.invoke(self, tex)
        c = self.ownerDocument.context
        name = self.attributes['name']
        options = self.attributes['options'] or {}

        # Create new subfloat class
        newclass = type(name, (subfloat,),
                                {'options':options,'counter':name})
        c.addGlobal(name, newclass)

        # Create new counter
        c.newcounter(name, resetby='figure', format='${%s.alph}' % name)

        # Create the float name macro
        c.newcommand(name+'name', 0, name)

class DeclareCaptionListOfFormat(Command):
    args = 'keyword code'

class subfloatname(Command):
    str = ''

class subfloat(Command):
    args = '[ toc ] [ caption ] self'
    counter = 'subfloat'
    options = {}

    def preParse(self, tex):
        """
        This is getting tricky.  The ContinuedFloat tells whether or not
        the counter should be incremented.  We save the lastvalue of the
        counter in the userdata so we can get it back here.

        """
        doc = self.ownerDocument
        c = doc.context

        if doc.userdata.getPath('packages/subfig/continued'):
            v = doc.userdata.getPath('packages/subfig/subfloats/%s/lastvalue' %
                                     self.tagName, 1)
            c.counters[self.counter].setcounter(v+1)
            doc.userdata.setPath('packages/subfig/continued', False)
        else:
            doc.userdata.setPath('packages/subfig/subfloats/%s/lastvalue' %
                                 self.tagName, c.counters[self.counter].value)

        return Command.preParse(self, tex)

    def invoke(self, tex):
        Command.invoke(self, tex)
        self.title = self.attributes['caption'] or self.attributes['toc']

    def ref():
        """
        This is a bit crazy.  We have to override ref so that we can
        add some new functionality.  The value normally gotten by ref
        is now held in subref.  The ref value also contains the ref value
        of the parent float node.

        """
        def fset(self, value):
            self.subref = value
        def fget(self):
            # Find the parent float of this subfloat
            node = self.parentNode
            while node is not None and not(isinstance(node, Float)):
                node = node.parentNode
            parentFloat = node

            # Add the float number to the ref value
            doc = self.ownerDocument
            frag = doc.createDocumentFragment()
            frag.append(parentFloat.caption.ref)
            frag.append(self.subref)
            return frag
        return locals()
    ref = property(**ref())

class subref(Command):
    args = '* label:idref'

class ContinuedFloat(Command):
    def invoke(self, tex):
        Command.invoke(self, tex)
        doc = self.ownerDocument
        c = doc.context
        c.counters['figure'].value -= 1
        doc.userdata.setPath('packages/subfig/continued', True)

class listsubcaptions(Command):
    pass

class captionsetup(Command):
    args = '[ type:str ] options:dict'



#!/usr/bin/env python

from plasTeX import Command, Environment
from plasTeX.Base.LaTeX.Quotations import verse

verse.args = '[ width:nox ]'

class altverse(verse):
    pass

class patverse(verse):
    pass

class patverse__star(verse):
    macroName = 'patverse*'



#!/usr/bin/env python

"""
Generic Page Template Renderer

This module contains a plasTeX renderer that uses various types of page
templates as the templating engine.  It also makes it possible to add
support for your own templating engines.

"""

import sys, os, re, plasTeX, shutil, string
from io import StringIO
from plasTeX.Renderers import Renderer as BaseRenderer
from plasTeX.Renderers.PageTemplate.simpletal import simpleTAL, simpleTALES
from plasTeX.Renderers.PageTemplate.simpletal.simpleTALES import Context as TALContext

log = plasTeX.Logging.getLogger()

# Support for Jinja2 templates
try: 
    from jinja2 import Environment, contextfunction
except ImportError:
    def jinja2template(s, encoding='utf8'):
        def renderjinja2(obj):
            return s
        return renderjinja2
else:
    try:
        import ipdb as pdb
    except ImportError:
        import pdb
        
    @contextfunction
    def debug(context):
        pdb.set_trace()

    def jinja2template(s, encoding='utf8'):
        env = Environment(trim_blocks=True, lstrip_blocks=True)
        env.globals['debug'] = debug

        def renderjinja2(obj, s=s):
            tvars = {'here':obj, 
                     'obj':obj,
                     'container':obj.parentNode,
                     'config':obj.ownerDocument.config,
                     'context':obj.ownerDocument.context,
                     'templates':obj.renderer}

            tpl = env.from_string(s)
            return tpl.render(tvars) 

        return renderjinja2

# Support for Python string templates
def stringtemplate(s):
    template = string.Template(s)
    def renderstring(obj):
        tvars = {'here':obj, 'self':obj, 'container':obj.parentNode,
                 'config':obj.ownerDocument.config, 'template':template,
                 'templates':obj.renderer, 'context':obj.ownerDocument.context}
        return template.substitute(tvars)
    return renderstring

# Support for Python string interpolations
def pythontemplate(s):
    template = s
    def renderpython(obj):
        tvars = {'here':obj, 'self':obj, 'container':obj.parentNode,
                 'config':obj.ownerDocument.config, 'template':template,
                 'templates':obj.renderer, 'context':obj.ownerDocument.context}
        return template % tvars
    return renderpython

# Support for ZPT HTML and XML templates
def htmltemplate(s):
    template = simpleTAL.compileHTMLTemplate(s)
    def renderhtml(obj):
        context = TALContext(allowPythonPath=1)
        context.addGlobal('here', obj)
        context.addGlobal('self', obj)
        context.addGlobal('container', obj.parentNode)
        context.addGlobal('config', obj.ownerDocument.config)
        context.addGlobal('context', obj.ownerDocument.context)
        context.addGlobal('template', template)
        context.addGlobal('templates', obj.renderer)
        output = StringIO()
        template.expand(context, output)
        return output.getvalue()
    return renderhtml

def xmltemplate(s):
    template = simpleTAL.compileXMLTemplate(s)
    def renderxml(obj):
        context = TALContext(allowPythonPath=1)
        context.addGlobal('here', obj)
        context.addGlobal('self', obj)
        context.addGlobal('container', obj.parentNode)
        context.addGlobal('config', obj.ownerDocument.config)
        context.addGlobal('context', obj.ownerDocument.context)
        context.addGlobal('template', template)
        context.addGlobal('templates', obj.renderer)
        output = StringIO()
        template.expand(context, output, docType=None, suppressXMLDeclaration=1)
        return output.getvalue()
    return renderxml

# Support for Cheetah templates
try:

   from Cheetah.Template import Template as CheetahTemplate
   from Cheetah.Filters import Filter as CheetahFilter
   class CheetahUnicode(CheetahFilter):
       def filter(self, val, **kw):
           return str(val)
   def cheetahtemplate(s):
       def rendercheetah(obj, s=s):
           tvars = {'here':obj, 'container':obj.parentNode,
                    'config':obj.ownerDocument.config,
                    'context':obj.ownerDocument.context,
                    'templates':obj.renderer}
           return CheetahTemplate(source=s, searchList=[tvars],
                                  filter=CheetahUnicode).respond()
       return rendercheetah

except (ImportError, AttributeError):

   def cheetahtemplate(s):
       def rendercheetah(obj):
           return str(s)
       return rendercheetah

# Support for Kid templates
try:

    from kid import Template as KidTemplate

    def kidtemplate(s):
        # Add namespace py: in
        s = '<div xmlns:py="http://purl.org/kid/ns#" py:strip="True">%s</div>' % s
        def renderkid(obj, s=s):
            tvars = {'here':obj, 'container':obj.parentNode,
                     'config':obj.ownerDocument.config,
                     'context':obj.ownerDocument.context,
                     'templates':obj.renderer}
            return str(KidTemplate(source=s, **tvars).serialize(fragment=1))
        return renderkid

except ImportError:

    def kidtemplate(s):
        def renderkid(obj):
            return str(s)
        return renderkid

# Support for Genshi templates
try:

    from genshi.template import MarkupTemplate as GenshiTemplate
    from genshi.template import TextTemplate as GenshiTextTemplate
    from genshi.core import Markup

    def markup(obj):
        return Markup(str(obj))

    def genshixmltemplate(s):
        # Add namespace py: in
        s = '<div xmlns:py="http://genshi.edgewall.org/" py:strip="True">%s</div>' % s
        template = GenshiTemplate(s)
        def rendergenshixml(obj):
            tvars = {'here':obj, 'container':obj.parentNode, 'markup':markup,
                     'config':obj.ownerDocument.config, 'template':template,
                     'context':obj.ownerDocument.context,
                     'templates':obj.renderer}
            return str(template.generate(**tvars).render(method='xml'))
        return rendergenshixml

    def genshihtmltemplate(s):
        # Add namespace py: in
        s = '<div xmlns:py="http://genshi.edgewall.org/" py:strip="True">%s</div>' % s
        template = GenshiTemplate(s)
        def rendergenshihtml(obj):
            tvars = {'here':obj, 'container':obj.parentNode, 'markup':markup,
                     'config':obj.ownerDocument.config, 'template':template,
                     'context':obj.ownerDocument.context,
                     'templates':obj.renderer}
            return str(template.generate(**tvars).render(method='html'))
        return rendergenshihtml

    def genshitexttemplate(s):
        template = GenshiTextTemplate(s)
        def rendergenshitext(obj):
            tvars = {'here':obj, 'container':obj.parentNode, 'markup':markup,
                     'config':obj.ownerDocument.config, 'template':template,
                     'context':obj.ownerDocument.context,
                     'templates':obj.renderer}
            return str(template.generate(**tvars).render(method='text'))
        return rendergenshitext

except ImportError:

    def genshixmltemplate(s):
        def rendergenshixml(obj):
            return str(s)
        return rendergenshixml

    def genshihtmltemplate(s):
        def rendergenshihtml(obj):
            return str(s)
        return rendergenshihtml

    def genshitexttemplate(s):
        def rendergenshitext(obj):
            return str(s)
        return rendergenshitext


def copytree(src, dest, symlink=None):
    """
    This is the same as shutil.copytree, but doesn't error out if the
    directories already exist.

    """
    for root, dirs, files in os.walk(src, True):
        for d in dirs:
            if d.startswith('.'):
                continue
            srcpath = os.path.join(root, d)
            destpath = os.path.join(dest, root, d)
            if symlink and os.path.islink(srcpath):
                if os.path.exists(destpath):
                    os.remove(destpath)
                os.symlink(os.readlink(srcpath), destpath)
            elif not os.path.isdir(destpath):
                os.makedirs(destpath)
                try:
                    shutil.copymode(srcpath, destpath)
                except: pass
                try:
                    shutil.copystat(srcpath, destpath)
                except: pass
        for f in files:
            if f.startswith('.'):
                continue
            srcpath = os.path.join(root, f)
            destpath = os.path.join(dest, root, f)
            if symlink and os.path.islink(srcpath):
                if os.path.exists(destpath):
                    os.remove(destpath)
                os.symlink(os.readlink(srcpath), destpath)
            else:
                shutil.copy2(srcpath, destpath)

class TemplateEngine(object):
    def __init__(self, ext, function):
        if not isinstance(ext, (list,tuple)):
            ext = [ext]
        self.ext = ext
        self.function = function
    def compile(self, *args, **kwargs):
        return self.function(*args, **kwargs)

class PageTemplate(BaseRenderer):
    """ Renderer for page template based documents """

    outputType = str
    fileExtension = '.xml'

    def __init__(self, *args, **kwargs):
        BaseRenderer.__init__(self, *args, **kwargs)
        self.loadedTheme = None
        self.engines = {}
        htmlexts = ['.html','.htm','.xhtml','.xhtm','.zpt','.pt']
        self.registerEngine('pt', None, htmlexts, htmltemplate)
        self.registerEngine('zpt', None, htmlexts, htmltemplate)
        self.registerEngine('zpt', 'xml', '.xml', xmltemplate)
        self.registerEngine('tal', None, htmlexts, htmltemplate)
        self.registerEngine('tal', 'xml', '.xml', xmltemplate)
        self.registerEngine('html', None, htmlexts, htmltemplate)
        self.registerEngine('xml', 'xml', '.xml', xmltemplate)
        self.registerEngine('python', None, '.pyt', pythontemplate)
        self.registerEngine('string', None, '.st', stringtemplate)
        self.registerEngine('kid', None, '.kid', kidtemplate)
        self.registerEngine('cheetah', None, '.che', cheetahtemplate)
        self.registerEngine('genshi', None, '.gen', genshihtmltemplate)
        self.registerEngine('genshi', 'xml', '.genx', genshixmltemplate)
        self.registerEngine('genshi', 'text', '.gent', genshitexttemplate)
        self.registerEngine('jinja2', None, '.jinja2', jinja2template)

    def registerEngine(self, name, type, ext, function):
        """
        Register a new type of templating engine

        Arguments:
        name -- the name of the engine
        type -- the type of output supported by the engine (e.g., html,
            xml, text, etc.)
        ext -- the file extensions associated with that template type
        function -- the function used to compile templates of that type

        """
        if not type:
            type = None
        key = (name, type)
        self.engines[key] = TemplateEngine(ext, function)

    def textDefault(self, node):
        """
        Default renderer for text nodes

        This method makes sure that special characters are converted to
        entities.

        Arguments:
        node -- the Text node to process

        """

        if not(getattr(node, 'isMarkup', None)):
            node = node.replace('&', '&amp;')
            node = node.replace('<', '&lt;')
            node = node.replace('>', '&gt;')
        return self.outputType(node)

    def loadTemplates(self, document):
        """ Load and compile page templates """
        themename = document.config['general']['theme']

        # Load templates from renderer directory and parent
        # renderer directories
        sup = list(type(self).__mro__)
        sup.reverse()
        themes = []
        for cls in sup:
            if cls is BaseRenderer or cls is object or cls is dict:
                continue
            cwd = os.path.dirname(sys.modules[cls.__module__].__file__)
            log.info('Importing templates from %s' % cwd)
            self.importDirectory(cwd)

            # Store theme location
            themes.append(os.path.join(cwd, 'Themes', themename))

            # Load templates configured by the environment variable
            templates = os.environ.get('%sTEMPLATES' % cls.__name__,'')
            for path in [x.strip() for x in templates.split(os.pathsep) if x.strip()]:
                log.info('Importing templates from %s' % path)
                self.importDirectory(path)
                themes.append(os.path.join(path, 'Themes', themename))

        # Load only one theme
        for theme in reversed(themes):
            if os.path.isdir(theme):
                log.info('Importing templates from %s' % theme)
                self.importDirectory(theme)
                self.loadedTheme = theme

                extensions = []
                for e in self.engines.values():
                    extensions += e.ext + [x+'s' for x in e.ext]

                if document.config['general']['copy-theme-extras']:
                    # Copy all theme extras
                    cwd = os.getcwd()
                    os.chdir(theme)
                    for item in os.listdir('.'):
                        if os.path.isdir(item):
                            if not os.path.isdir(os.path.join(cwd,item)):
                                os.makedirs(os.path.join(cwd,item))
                            copytree(item, cwd, True)
                        elif os.path.splitext(item)[-1].lower() not in extensions:
                            shutil.copy(item, os.path.join(cwd,item))
                    os.chdir(cwd)

                break

    def render(self, document):
        """ Load templates and render the document """
        self.loadTemplates(document)
        BaseRenderer.render(self, document)

    def importDirectory(self, templatedir):
        """
        Compile all ZPT files in the given directory

        Templates can exist in two different forms.  First, a template
        can be a file unto itself.  If an XML template is desired,
        the file should have an extension of .xml, .xhtml, or .xhtm.
        If an HTML template is desired, the files should have an
        extension of .zpt, .html, or .htm.  You can also configure
        your own page templates with their own extensions.

        If you have many small templates, or a template that corresponds
        to more than one macro, you can use a multiple ZPT file.  A
        multiple ZPT file contains directives within it to delimit
        individual page templates as well as specify which macros they
        correspond to and what type of template they are (i.e. XML or
        HTML).

        Required Arguments:
        templatedir -- the directory to search for template files

        """
        # Create a list for resolving aliases
        self.aliases = {}

        enames = {}
        for key, value in self.engines.items():
            for i in value.ext:
                enames[i+'s'] = key[0]

        singleenames = {}
        for key, value in self.engines.items():
            for i in value.ext:
                singleenames[i] = key[0]

        if templatedir and os.path.isdir(templatedir):
            files = os.listdir(templatedir)

            # Compile multi-pt files first
            for f in files:
                ext = os.path.splitext(f)[-1]
                f = os.path.join(templatedir, f)

                if not os.path.isfile(f):
                    continue

                # Multi-pt files
                if ext.lower() in enames:
                    self.parseTemplates(f, {'engine':enames[ext.lower()]})

            # Now compile macros in individual files.  These have
            # a higher precedence than macros found in multi-pt files.
            for f in files:
                basename, ext = os.path.splitext(f)
                f = os.path.join(templatedir, f)

                if not os.path.isfile(f):
                    continue

                options = {'name':basename}

                for value in self.engines.values():
                    if ext in value.ext:
                        options['engine'] = singleenames[ext.lower()]
                        self.parseTemplates(f, options)
                        del options['engine']
                        break

        if self.aliases:
           log.warning('The following aliases were unresolved: %s'
                       % ', '.join(list(self.aliases.keys())))

    def setTemplate(self, template, options):
        """
        Compile template and set it in the renderer

        Required Arguments:
        template -- the content of the template to be compiled
        options -- dictionary containing the name (or names) and type
            of the template

        """

        # Get name
        try:
            names = options['name'].split()
            if not names:
                names = [' ']
        except KeyError:
            raise ValueError('No name given for template')

        # If an alias was specified, link the names to the
        # already specified template.
        if 'alias' in options:
            alias = options['alias'].strip()
            for name in names:
                self.aliases[name] = alias
            if ''.join(template).strip():
                log.warning('Both an alias and a template were specified for: %s' % ', '.join(names))

        # Resolve remaining aliases
        for key, value in list(self.aliases.items()):
            if value in self:
                self[key] = self[value]
            self.aliases.pop(key)

        if 'alias' in options:
            return

        # Compile template and add it to the renderer
        template = ''.join(template).strip()
        ttype = options.get('type')
        if ttype is not None:
            ttype = ttype.lower()
        engine = options.get('engine','zpt').lower()

        templateeng = self.engines.get((engine, ttype),
                            self.engines.get((engine, None)))

        try:
            template = templateeng.compile(template)
        except Exception as msg:
            raise ValueError('Could not compile template "%s"' % names[0])

        for name in names:
            self[name] = template

    def parseTemplates(self, filename, options={}):
        """
        Parse templates from the file and set them in the renderer

        Required Arguments:
        filename -- file to parse templates from

        Keyword Arguments:
        options -- dictionary containing initial parameters for templates
            in the file

        """
        num_templates = 0
        template = []
        options = options.copy()
        defaults = options.copy()
        name = None
        if not options or 'name' not in options:
            f = open(filename, 'r')
            for i, line in enumerate(f):
                # Found a meta-data command
                if re.match(r'(default-)?\w+:', line):

                    # Purge any awaiting templates
                    if template:
                        try:
                            num_templates += 1
                            self.setTemplate(''.join(template), options)
                        except ValueError as msg:
                            print('ERROR: %s at line %s in file %s' % (msg, i, filename))
                        options = defaults.copy()
                        template = []

                    # Done purging previous template, start a new one
                    name, value = line.split(':', 1)
                    name = name.strip()
                    value = value.rstrip()
                    while value.endswith('\\'):
                        value = value[:-1] + ' '
                        for line in f:
                            value += line.rstrip()
                            break

                    value = re.sub(r'\s+', r' ', value.strip())
                    if name.startswith('default-'):
                        name = name.split('-')[-1]
                        defaults[name] = value
                        if name not in options or num_templates == 0:
                            options[name] = value
                    else:
                        options[name] = value
                    continue

                if template or (not(template) and line.strip()):
                    template.append(line)
                elif not(template) and 'name' in options:
                    template.append('')

        else:
            with open(filename, encoding='utf-8') as f:
                template = f.readlines()

        # Purge any awaiting templates
        if template:
            try:
                self.setTemplate(''.join(template), options)
            except ValueError as msg:
                print('ERROR: %s in template %s in file %s' % (msg, ''.join(template), filename))

        elif name and not(template):
            self.setTemplate('', options)

    def processFileContent(self, document, s):
        # Add width, height, and depth to images
        s = re.sub(r'&amp;(\S+)-(width|height|depth);(?:&amp;([a-z]+);)?',
                   self.setImageData, s)

        # Convert characters >127 to entities
        if document.config['files']['escape-high-chars']:
            s = list(s)
            for i, item in enumerate(s):
                if ord(item) > 127:
                    s[i] = '&#%.3d;' % ord(item)
            s = ''.join(s)
        
        return BaseRenderer.processFileContent(self, document, s)

    def setImageData(self, m):
        """
        Substitute in width, height, and depth parameters in image tags

        The width, height, and depth parameters aren't known until after
        all of the output has been generated.  We have to post-process
        the files to insert this information.  This method replaces
        the &filename-width;, &filename-height;, and &filename-depth;
        placeholders with their appropriate values.

        Required Arguments:
        m -- regular expression match object that contains the filename
            and the parameter: width, height, or depth.

        Returns:
        replacement for entity

        """
        filename, parameter, units = m.group(1), m.group(2), m.group(3)

        try:
            img = self.imager.images.get(filename, self.vectorImager.images.get(filename, self.imager.staticimages.get(filename)))
            if img is not None and getattr(img, parameter) is not None:
                if units:
                    return getattr(getattr(img, parameter), units)
                return str(getattr(img, parameter))
        except KeyError: pass

        return '&%s-%s;' % (filename, parameter)


# Set Renderer variable so that plastex will know how to load it
Renderer = PageTemplate




#!/usr/bin/env python

from plasTeX.ConfigManager.Generic import GenericOption, GenericParser, GenericArgument
from plasTeX.ConfigManager import InvalidOptionError

class IntegerParser(GenericParser): pass

class IntegerOption(IntegerParser, GenericOption):
   """ Integer configuration option """

   synopsis = 'n'

   def _hasFollowingArgument(self, args, delim):
      """ Return boolean indicating the existence of another argument """
      return 0

   def cast(self, data):
      name = self.name
      if self.actual: name = self.actual
      if data is None: return
      try: return int(data)
      except: raise InvalidOptionError(name, data, type='int')


class IntegerArgument(GenericArgument, IntegerOption):
   """ Integer command-line argument """



#!/usr/bin/env python

"""
C.3.1 Making Sentences (p170)

"""

import time
from plasTeX import Command, Environment
from plasTeX.Logging import getLogger

#
# Quotes
#
# '          Apostrophe
# `text'     Single quotes
# ``text''   Double quotes

#
# Dashes
#
# -      Intra-word
# --     Number-range
# ---    Punctuation

#
# Spacing
#

class SmallSpace(Command):
    macroName = ','
    str = '\u2009'

class InterWordSpace(Command):
    macroName = ' '
    str = '\u0020'

class NoLineBreak(Command):
    macroName = 'active::~'
    str = '\u00A0'

class EndOfSentence(Command):
    macroName = '@'
    str = ''

class frenchspacing(Command): 
    pass

class nonfrenchspacing(Command):
    pass

#
# Special characters
#

class Dollar(Command):
    macroName = '$'
    str = '$'

class Percent(Command):
    macroName = '%'
    str = '%'

class LeftBrace(Command):
    macroName = '{'
    str = '{'

class Underscore(Command):
    macroName = '_'
    str = '_'

class Ampersand(Command):
    macroName = '&'
    str = '&'

class HashMark(Command):
    macroName = '#'
    str = '#'

class RightBrace(Command):
    macroName = '}'
    str = '}'

#
# Logos
#

class LaTeX(Command):
    pass

class TeX(Command):
    pass

#
# Misc
#

class emph(Command):
    args = 'self'

class em(Environment):
    pass
    
class textsubscript(Command):
     args = 'self'
 
class textsuperscript(Command):
    args = 'self'




#!/usr/bin/env python

"""
C.3.2 Making Paragraphs (p171)

"""

from plasTeX.Tokenizer import Other
from plasTeX import Command, Environment, DimenCommand
from plasTeX.Logging import getLogger

class noindent(Command):
    pass

class indent(Command):
    pass

# Defined in TeX
#class par(Command):
#    pass

#
# Style Parameters
#

class textwidth(DimenCommand):
    value = DimenCommand.new('6.5in')

class columnwidth(DimenCommand):
    value = DimenCommand.new('6.5in')

class linewidth(DimenCommand):
    value = DimenCommand.new('6.5in')

class parindent(DimenCommand):
    value = DimenCommand.new(0)

class baselineskip(DimenCommand):
    value = DimenCommand.new('12pt')

class baselinestretch(Command):
    str = '1'

class parskip(DimenCommand):
    value = DimenCommand.new(0)



#!/usr/bin/env python

from plasTeX.ConfigManager.Generic import GenericOption, GenericParser, GenericArgument
from plasTeX.ConfigManager import InvalidOptionError


class FloatParser(GenericParser): pass

class FloatOption(FloatParser, GenericOption):
   """ Float configuration option """

   synopsis = 'num'

   def cast(self, data):
      name = self.name
      if self.actual: name = self.actual
      if data is None: return
      try: return float(data)
      except: raise InvalidOptionError(name, data, type='float')


class FloatArgument(GenericArgument, FloatOption):
   """ Float command-line argument """



#!/usr/bin/env python

"""
C.13.2 Space (p216)

"""

from plasTeX import Command, Environment, DimenCommand
from plasTeX.Logging import getLogger

class hspace(Command):
    args = '* len:dimen'

class vspace(Command):
    args = '* len:dimen'

class phantom(Command):
    args = 'text:str'

class bigskip(Command):
    pass

class medskip(Command):
    pass

class smallskip(Command):
    pass

class bigskipamount(DimenCommand):
    value = DimenCommand.new('24pt')

class medskipamount(DimenCommand):
    value = DimenCommand.new('12pt')

class smallskip(DimenCommand):
    value = DimenCommand.new('6pt')

class addvspace(Command):
    args = 'len:dimen'

class hfill(Command):
    pass

class vfill(Command):
    pass



#!/usr/bin/env python

"""
C.14.1 The picture Environment (p219)

"""

from plasTeX import Command, Environment, DimenCommand
from plasTeX.Logging import getLogger

class qbeziermax(Command):
    str = '250'

class unitlength(DimenCommand):
    value = DimenCommand.new('1pt')

class picture(Environment):
    args = '( dimension:str ) ( offset:str )'
    blockType = True
    captionable = True

#   def digest(self, tex):
#       result = Environment.digest(self, tex)
#       import pdb
#       pdb.set_trace()
#       return result

    def paragraphs(self):
        pass

    class put(Command):
        args = '( coord:str ) object'

    class multiput(Command):
        args = '( coord:str ) ( incr:str ) num:int object'

    class qbezier(Command):
        args = '[ num:int ] ( coord1:str ) ( coord2:str ) ( coord3:str )'

    class graphpaper(Command):
        args = '[ spacing ] ( coord:str ) ( dimen:str )'

    class makebox(Command):
        args = '( dimen:str ) [ pos:str ] text'

    class framebox(Command):
        args = '( dimen:str ) [ pos:str ] text'

    class dashbox(Command):
        args = 'dashdimen:str ( dimen:str ) [ pos:str ] text'

    class line(Command):
        args = '( slope ) dimen'

    class vector(Command):
        args = '( slope ) dimen'

    class shortstack(Command):
        args = '[ pos:str ] col'

    class circle(Command):
        args = '* diam'

    class oval(Command):
        args = '[ radius:str ] ( dimen:str ) [ part:str ]' 

    class frame(Command):
        args = 'object'

    class savebox(Command):
        args = 'name:cs ( dimen:str ) [ pos:str ] text'

    class thinlines(Command):
        pass

    class thicklines(Command):
        pass

    class linethickness(Command):
        args = 'len:dimen'
    



#!/usr/bin/env python

"""
Implementation of the nameref package

"""

from plasTeX import Command, Environment

class nameref(Command):
    args = 'label:idref'
    
class Nameref(Command):
    args = 'label:idref'


#!/usr/bin/env python

from plasTeX import Command

class fancypagestyle(Command):
    args = 'style'

class fancyhead(Command):
    args = '[ pos ] text'

class fancyfoot(fancyhead):
    pass

class fancyhf(Command):
    args = 'text'

class rightmark(Command):
    pass

class leftmark(Command):
    pass

class chaptermark(Command):
    pass

class sectionmark(Command):
    pass

class markboth(Command):
    pass

class markright(Command):
    pass



#!/usr/bin/env python

import os, re, glob
from plasTeX.Imagers import Imager as _Imager

class DVISVGM(_Imager.VectorImager):
    """ Imager that uses dvisvgm """
    fileExtension = '.svg'
    verification = 'dvisvgm --help'
    compiler = 'latex'

    def executeConverter(self, output):
        rc = 0
        open('images.dvi', 'w').write(output.read())
        page = 1
        while 1:
            filename = 'img%d.svg' % page
            rc = os.system('dvisvgm --exact --scale=1.6 --no-fonts --output=%s --page=%d images.dvi' % (filename, page))
            if rc:
                break

            # dvisvgm always puts "-<page-number>" on each file.  Get rid of it.
            try: os.rename(glob.glob(filename+'-*')[0], filename)
            except IndexError: break

            if not open(filename).read().strip():
                os.remove(filename)
                break
            page += 1
            if page > len(self.images):
                break
        return rc, None

Imager = DVISVGM



#!/usr/bin/env python

"""

plasTeX.Package.amsthm.py: Partial support for `amsthm.sty` LaTeX Package.

This python package provides support for


* Command: `\\newtheorem`
* Command: `\\newtheoremstyle`
* Command: `\\theoremstyle`
* Environment: `proof`

Keep in mind that when you define and use a new theorem-like environment:

    \\newtheorem{theorem}{Theorem}
    \\begin{theorem}[Euclid's] This is a Theorem. \end{theorem}

this package translates the above construction to simplified LaTeX code (as opposed to a `theorem` environment that can be styled throught the renderer.)

I chose this instead of relying on the renderer because the user can specify everything, from the names to the header fonts, to the colors, to the body font, etc. I don't think the rendering mechanism can cope with environments of user defined names and user defined styles.

It supports the starred version (without numbers) of `\\newtheorem`

This package keeps a list of user defined styles in the `userdata` space under `packages/amsthm/styles`. This userdata is prepopulated with (a simplified version of) the  default styles in amsthm.sty: plain, remark, and definition.

It also keeps a list of theorem-like environments under `packages/amsthm/theorems`.

Detailed support:

1. `\\newtheorem * {name}[shared counter]{Header}[reset by]` is supported. It creates an environment `\\begin{name}[optional note] .. \end{name}`, which translates on invocation into simpler LaTeX.

2. `\\newtheoremstyle {name} {above space} {below space} {main body font} {indent amount} {header font} {punctuation hafter head} {space between head and body} {header spec}` is partially supported---it currently ignores everything but main body font and header font. The header spec is hardcoded and equivalent to `\\thmname{#1}\\thmnumber{ #2}\\thmnote{ (3)}`. Read the `amsthm` manual for more information on defining theorem styles. There's a list of defined styles in `userdata` space, in `packages/amsthm/styles`.

3. `\\theoremstyle{name}` Set the current theorem style to `name`. Not checked for errors.

4 `\\begin{proof} .. \end{proof}` Translated into simpler LaTeX.

5. `\swapnumbers`  Ignored & Unsupported.

TODO & NOT FULLY WORKING:

1. I can't get `\label` to  reference properly the correct statement.


2. For some reason, I can't get

        \\newtheorem{theorem}{Theorem}
        \\begin{theorem} Statement \end{theorem}

    to wrap the Statement in a span. Currently it gives you something like:

        <p><b>Theorem 1</b> <i>Statement</i><p>

    and I would like something like:

        <span class="statement"><p><b>Theorem 1</b> <i>Statement</i><p></span>

3. Support of `header spec`.


Example document:

    \documentclass{article}
    \\usepackage{amsthm}
    \\usepackage{color}

    \\newtheoremstyle{redPlain}  {\\bigskipamount}
                                {\\bigskipamount}
                                {\itshape}
                                {}
                                {\color[cmyk]{0,1.00,0.65,0.34}\\bfseries}
                                {:}
                                {1em}
                                {\\thmname{#1}\\thmnumber{ #2}\\thmnote{ (#3)}}
    \\theoremstyle{redPlain}
    \\newtheorem*{starred}{Starred}
    \\newtheorem{normal}{Normal}
    \\newtheorem{normalshared}[normal]{Normal And Shared}
    \\theoremstyle{plain}
    \\newtheorem{within}{Within}[section]
    \\newtheorem{withinshared}[within]{Within And Shared}
    \\begin{document}

    \section{A section}
    \\begin{starred}[Note]
        An starred theorem
    \end{starred}
    \\begin{normal}
        A normal theorem
    \end{normal}
    \\begin{normalshared}[Note]
        A normal theorem, with shared counter.
    \end{normalshared}
    \\begin{within}
        A theorem numbered within sections
    \end{within}
    \\begin{withinshared}[Note]
        A theorem with counter shared with a theorem numbered within sections
    \end{withinshared}
    \section{Another section}
    \\begin{starred}
        An starred theorem
    \end{starred}
    \\begin{normal}
        A normal theorem
    \end{normal}
    \\begin{normalshared}
        A normal theorem, with shared counter.
    \end{normalshared}
    \\begin{within}
        A theorem numbered within sections
    \end{within}
    \\begin{withinshared}\label{T:1}
        A theorem with counter shared with a theorem numbered within sections
    \end{withinshared}
    \end{document}


"""

import plasTeX


class swapnumbers(plasTeX.Command):
    pass


def ProcessOptions(options, document):
    context = document.context
    context.newenvironment("proof", 1, [
        u"\\par\\noindent{\\normalfont\\itshape #1.}\\enspace",
        r""
    ], opt = u"\\proofname")
    document.userdata.setPath('packages/amsthm/styles',{
        'plain' : {
            'name': 'plain',
            'above': u"0pt",
            'below': u"0pt",
            'bodyfont': u"\\itshape",
            'indentamount': u'0pt',
            'headfont': u"\\bfseries",
            'punctuation': u".",
            'between': u' ',
            'headspec': None
        },
        'definition' : {
            'name': 'definition',
            'above': u"0pt",
            'below': u"0pt",
            'bodyfont': u"",
            'indentamount': u'0pt',
            'headfont': u"\\bfseries",
            'punctuation': u".",
            'between': u' ',
            'headspec': None
        },
        'remark' : {
            'name': u'remark',
            'above': u'0pt',
            'below': u'0pt',
            'bodyfont': u"",
            'indentamount': '0pt',
            'headfont': u"\\bfseries",
            'punctuation': u".",
            'between': u' ',
            'headspec': None
        },

    })
    document.userdata.setPath('packages/amsthm/theorems',[])
    document.userdata.setPath('packages/amsthm/currentstyle', 'plain')



class theoremstyle(plasTeX.Command):
    args = 'style:str'
    def invoke(self, tex):
        self.parse(tex)
        self.ownerDocument.userdata.setPath('packages/amsthm/currentstyle', self.attributes['style'])


class newtheoremstyle(plasTeX.Command):
    args = 'name above below bodyfont indentamount headfont punctuation between headspec'
    def invoke(self, tex):
        self.parse(tex)
        d = self.ownerDocument.userdata.getPath("packages/amsthm/styles")
        s = {}
        for k, v in self.attributes.items():
            s[str(k)]=tex.source(v)
        d[tex.source(self.attributes['name'])] = s


class newtheorem(plasTeX.Command):
    args = ' * name:str [ shared:str ] header:str [ parent:str] '
    
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        name = str(a['name'])
        header = a['header']
        star = a['*modifier*'] == '*'
        parent = a['parent']
        shared = a['shared']
        style = self.ownerDocument.userdata.getPath('packages/amsthm/currentstyle')

        l = self.ownerDocument.userdata.getPath('packages/amsthm/theorems')
        l += [name]


        if star:
            thecounter = None
        else:
            if parent and not shared:
                self.ownerDocument.context.newcounter(name, initial=0, resetby=parent)
                self.ownerDocument.context.newcommand("the"+name, 0,
                                        "\\arabic{%s}.\\arabic{%s}"%(parent,name))
                thecounter = name
            elif shared:
                thecounter = shared
            else:
                thecounter = name
                self.ownerDocument.context.newcounter(name, initial=0)
                self.ownerDocument.context.newcommand("the"+name, 0, "\\arabic{%s}" %name)

        data = {
                'macroName': name,
                'counter': thecounter,
                'thehead': header,
                'thename': name,
                'labelable': True,
                'forcePars': True,
                'thestyle': style
            }
        th = type(name, (theoremCommand,), data)
        self.ownerDocument.context.addGlobal(name, th)


class theoremCommand(plasTeX.Environment):
    args = '[ note:chr ]'
    blockType = True
    
    def invoke(self, tex):
        # self.style['class'] = u'amsthm '+self.thename
        if self.macroMode == self.MODE_BEGIN:
            note = tex.readArgument('[]')
            
            if note:
                note = tex.source(note)
                
            if self.counter:
# Done automagically:  self.ownerDocument.context.counters[self.counter].stepcounter()
                self.ownerDocument.context.currentlabel = self
                self.style['id'] = self.id

            style_dict = self.ownerDocument.userdata.getPath(
                "packages/amsthm/styles/"+self.thestyle
            )
            head_style = style_dict['headfont']
            note_opener = u'('
            note_closer = u')'
            punctuation = style_dict['punctuation'] if style_dict['punctuation'] else u"."
            body_style = style_dict['bodyfont']
            # between = style_dict['between']
            between = u' '

            s = u'{'+head_style+u"{}"+self.thehead

            if self.counter:
                s += u'~\\the'+self.counter

            if note:
                s += u'{} ' + note_opener + note + note_closer

            s += punctuation+between+'}'+body_style
            self.ownerDocument.context.push(self)
            tex.input(s)
            super(theoremCommand, self).invoke(tex)
            
        elif self.macroMode == self.MODE_END:
            self.ownerDocument.context.pop(self)
            return




#!/usr/bin/env python
import re
from plasTeX.Renderers.PageTemplate import Renderer as _Renderer
from plasTeX import Command
try:
    from lxml import etree
except ImportError:
    have_lxml = False
else:
    have_lxml = True
xns = {'d': 'http://docbook.org/ns/docbook'}

def drop_tag(elem):
    """
    Remove the tag, but not its children or text.
    The children and text are merged into the parent.
    """
    parent = elem.getparent()
    previous = elem.getprevious()
    if elem.text and isinstance(elem.tag, str):
        if previous is None:
            parent.text = (parent.text or '') + elem.text
        else:
            previous.tail = (previous.tail or '') + elem.text
    if elem.tail:
        if len(elem):
            last = elem[-1]
            last.tail = (last.tail or '') + elem.tail
        elif previous is None:
            parent.text = (parent.text or '') + elem.tail
        else:
            previous.tail = (previous.tail or '') + elem.tail
    index = parent.index(elem)
    parent[index:index + 1] = elem[:]

def clean_para(tree, name):
    for elem in tree.findall('.//d:%s' % name, namespaces=xns):
        e = elem.findall('d:para', namespaces=xns)
        if e:
            for tag in e:
                drop_tag(tag)
    return tree

def get_see(term):
    see = None
    seealso = None
    if term.find('|') != -1:
        term, fmt = term.split('|')
        if fmt.find('seealso') != -1:
            seealso = fmt
        elif fmt.find('see') != -1:
            see = fmt

    return term, see, seealso

def parse_indexentry(s):
    term = s
    sortstr = None
    if term.find('@') != -1:
        term, sortstr = term.split('@')
    term, see, seealso = get_see(term)
    return (term, sortstr, see, seealso)

class index(Command):
    args = 'argument:str'

    def invoke(self, tex):
        Command.invoke(self, tex)
        entry = self.attributes['argument']
        if entry.find('!') != -1:
            primary, secondary = entry.split('!')

            primary, prisort, see, seealso= parse_indexentry(primary)
            if see or seealso:
                secondary, secsort, _, _ = parse_indexentry(secondary)
            else:
                secondary, secsort, see, seealso = parse_indexentry(secondary)
        else:
            primary, prisort, see, seealso = parse_indexentry(entry)

        self.data = {
            'primary': primary,
            'secondary':secondary,
            'prisort': prisort,
            'secsort': secsort,
            'see': see,
            'seealso': seealso,
            }

class DocBook(_Renderer):
    """ Renderer for DocBook documents """
    fileExtension = '.xml'
    imageTypes = ['.png','.jpg','.jpeg','.gif']
    vectorImageTypes = ['.svg']

    def cleanup(self, document, files, postProcess=None):
        res = _Renderer.cleanup(self, document, files, postProcess=postProcess)
        return res

    def processFileContent(self, document, s):
        s = _Renderer.processFileContent(self, document, s)

        if have_lxml:
            tree = etree.fromstring(s)
            for name in ['itemizedlist', 'table', 'term', 'para']:
                tree = clean_para(tree, name)
            s = etree.tostring(tree, encoding='unicode')
        else:
            s = re.sub(r'</partintro>\s*<partintro>','', s, flags=re.I)
            s = re.sub(r'<para>\s*(<articleinfo>)', r'\1', s, flags=re.I)
            s = re.sub(r'(</articleinfo>)\s*</para>', r'\1', s, flags=re.I)
            s = re.sub(r'<para>\s*</para>', '', s, flags=re.I)

            for name in ['itemizedlist', 'term', 'para']:
                s = re.sub(r'(<%s>)\s*<para>' % name, r'\1', s, flags=re.I)
                s = re.sub(r'</para>\s*(</%s>)' % name, r'\1', s, flags=re.I)

        return s

Renderer = DocBook



#!/usr/bin/env python

"""

"""

from plasTeX import Command, Environment


class center(Environment): 
    blockType = True

class centering(center):
    blockType = True

class flushleft(Environment):
    blockType = True

class raggedright(flushleft):
    blockType = True

class flushright(Environment):
    blockType = True

class raggedleft(flushright):
    blockType = True

class raggedbottom(Environment):
    pass



#!/usr/bin/env python

from plasTeX.ConfigManager.Generic import GenericArgument
from plasTeX.ConfigManager.Boolean import BooleanOption
from plasTeX.ConfigManager import InvalidOptionError, COMMANDLINE

class CountedOption(BooleanOption):
   """
   Counted configuration option

   This option is just like a boolean option except that the value
   of the option is the number of times that the option has been
   specified.  This is commonly used to set a verbosity or debugging
   level.

   """

   def cast(self, data):
      if data is None: return

      values = {'on':1,'off':0,'true':1,'false':0,'yes':1,'no':0}

      # If this is from the command-line, increment or decrement
      if self.source & COMMANDLINE:
         initdata = self.data
         if self.data is None:
            initdata = 0
         if data:
            return max(0, initdata + 1)
         else:
            return max(0, initdata - 1)

      # If this is from any other source, set explicitly
      try:
         return max(0,int(data))
      except:
         try: return values[str(data).lower()]
         except: pass
      name = self.name
      if self.actual: name = self.actual
      raise InvalidOptionError(name, data, type='counted')

   def __str__(self):
      """ Return string representation of the current option value """
      value = self.getValue()
      if value == 1:
         return 'on'
      elif value == 0:
         return 'off'
      return str(value)

   def __repr__(self):
      """ Return command-line syntax as entered """
      if self.actual: return self.actual
      options = self.getPossibleOptions()
      negative = [x.replace('!','',1) for x in options if x.startswith('!')]
      positive = [x for x in options if not x.startswith('!')]
      if self.data and positive:
         return ' '.join(self.data*[positive[0]])
      elif not(self.data) and negative:
         return negative[0]
      return ''


class CountedArgument(GenericArgument, CountedOption):
   """ Counted command-line argument """



#!/usr/bin/env python

"""
ZPT - Zope Page Template Renderer

This module contains a plasTeX renderer that uses Zope Page Templates
(actually simpleTAL) as the templating engine.

NOTE: This module isn't really needed any more.  The PageTempate 
renderer does everything that this module did and more.  This is 
just here for backwards compatibility.

"""

from plasTeX.Renderers.PageTemplate import Renderer

class ZPT(Renderer):
    pass

Renderer = ZPT



#!/usr/bin/env python

"""
C.6.1 Quotations and Verse (p184)

"""

from plasTeX import Command, Environment
from plasTeX.Logging import getLogger


class quote(Environment):
    blockType = True

class quotation(Environment):
    blockType = True

class verse(Environment):
    blockType = True

# TODO figure out why putting this in plasTeX.Renderers.Gerby causes things to malfunction
class slogan(Environment):
  blockType = True

class history(Environment):
  blockType = True

class reference(Environment):
  blockType = True



#!/usr/bin/env python

from plasTeX import Command, Environment, log

# Map of CJK encoding names to Python encoding names
encodings = {
    'Bg5': 'big5',
    'GB': '',
    'GBt': '',
    'JIS': 'euc_jp',
    'SJIS': 'shift_jis',
    'KS': '',
    'UTF8': 'utf_8',
}

class CJK(Environment):
    args = '* [ fontencoding ] encoding args'

class CJKchar(Command):
    args = '[ encoding:str ] char1:int char2:int'
    @property
    def str(self):
        a = self.attributes
        enc = encodings[a['encoding'].strip()]
        if not enc:
            log.warning('Unknown encoding: %s' % a['encoding'])
            raise AttributeError('str')
        return str(chr(a['char1'])+chr(a['char2'])).encode(enc)

class Str(Command):
    args = 'char1:int char2:int'
    @property
    def str(self):
        return chr(a['char1'])+chr(a['char2']) 

class CJKcaption(Command):
    args = 'caption:str'

class CKJenc(Command):
    args = 'encoding:str'

class CJKfamily(Command):
    args = 'family:str'

class CJKencfamily(Command):
    args = '[ fontencoding:str ] encoding:str family:str'

class CJKfontenc(Command):
    args = 'encoding:str fontencoding:str'

class CJKkern(Command):
    pass

class CJKglue(Command):
    pass

class CJKtilde(Command):
    pass

class nbs(Command):
    pass

class standardtilde(Command):
    pass

class CJKspace(Command):
    pass

class CJKnospace(Command):
    pass

class CJKhanja(Command):
    pass

class CJKhanjul(Command):
    pass

class CJKtolerance(Command):
    pass

class CJKuppercase(Command):
    pass

class Bg5text(Environment):
    pass

class SJIStext(Environment):
    pass

class CJKCJKchar(Command):
    pass

class CJKhangulchar(Command):
    pass

class CJKlatinchar(Command):
    pass

class CJKbold(Command):
    pass

class CJKnormal(Command):
    pass

class CJKsymbol(Command):
    pass

class CJKsymbols(Command):
    pass

class CJKboldshift(Command):
    pass

class CJKaddEncHook(Command):
    args = 'encoding:str body'



#!/usr/bin/env python

from plasTeX import Command, Environment
from plasTeX.Base.LaTeX.Floats import Float, Caption

class newfloat(Command):
    args = 'name:str pos:str capfile:str [ reset:str ]'
    def invoke(self, tex):
        Command.invoke(self, tex)
        name = str(self.attributes['name'])

        # Create the float class and the caption class
        floatcls = type(name, (Float,), {})
        captioncls = type('caption', (Caption,),
                                  {'macroName':'caption', 'counter':name})
        floatcls.caption = captioncls
        c = self.ownerDocument.context
        c.addGlobal(name, floatcls)

        # Create a counter
        resetby = self.attributes['reset'] or 'chapter'
        c.newcounter(name, resetby, 0, format='${the%s}.${%s}' % (resetby,name))

        # Create the float name macro
        c.newcommand(name+'name', 0, name)


class floatstyle(Command):
    args = 'style:str'

class restylefloat(Command):
    args = 'float:str'

class floatname(Command):
    args = 'float:str name:str'
    def invoke(self, tex):
        Command.invoke(self, tex)
        float = str(self.attributes['float'])
        name = self.attributes['name']
        c = self.ownerDocument.context
        c.newcommand(float+'name', 0, name)

class floatplacement(Command):
    args = 'float:str pos:str'

class listof(Command):
    args = 'float:str title'



import os
from plasTeX.ConfigManager import *
from plasTeX.DOM import Node

config = ConfigManager()

section = config.add_section('html5')

config.add_category('html5', 'Html5 renderer Options')

section['extra-css'] = MultiOption(
    """ Extra css files to use """,
    options='--extra-css',
    category='html5',
    default='',
)

section['extra-js'] = MultiOption(
    """ Extra javascript files to use """,
    options='--extra-js',
    category='html5',
    default='',
)

section['theme-css'] = StringOption(
    """ Theme css file""",
    options='--theme-css',
    category='html5',
    default='green',
)

section['use-theme-css'] = BooleanOption(
    """ Use theme css """,
    options='--use-theme-css !--no-theme-css',
    category='html5',
    default=True,
)

section['use-theme-js'] = BooleanOption(
    """ Use theme javascript """,
    options='--use-theme-js !--no-theme-js',
    category='html5',
    default=True,
)

section['display-toc'] = BooleanOption(
    """ Display table of contents on each page """,
    options='--display-toc !--no-display-toc',
    category='html5',
    default=True,
)

section['localtoc-level'] = IntegerOption(
    """ Create local toc above this level """,
    options='--localtoc-level',
    category='html5',
    default=Node.DOCUMENT_LEVEL-1,
)

section['breadcrumbs-level'] = IntegerOption(
    """ Create breadcrumbs from this level """,
    options='--breadcrumbs-level',
    category='html5',
    default=-10,
)

section['use-mathjax'] = BooleanOption(
    """ Use mathjax """,
    options='--use-mathjax !--no-mathjax',
    category='html5',
    default=True,
)

section['mathjax-url'] = StringOption(
    """ Url of the MathJax lib """,
    options='--mathjax-url',
    category='html5',
    default='http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS_CHTML',
)

section['mathjax-dollars'] = BooleanOption(
    """ Use single dollars as math delimiter for mathjax """,
    options='--dollars !--no-dollars',
    category='html5',
    default=False,
)

section['filters'] = MultiOption(
    """Comma separated list of commands to invoke on each output page.""",
    options='--filters',
    category='html5',
    default='',
)

section['tikz-compiler'] = StringOption(
    """ LaTeX compiler for TikZ pictures """,
    options='--tikz-compiler',
    category='html5',
    default='pdflatex',
)

section['tikz-converter'] = StringOption(
    """ PDF to SVG converter for tikz and tikz-cd """,
    options='--tikz-converter',
    category='html5',
    default='pdf2svg',
)

section['tikz-template'] = StringOption(
    """ Jinja2 template file for tikz """,
    options='--tikz-template',
    category='html5',
    default='',
)

section['tikz-cd-template'] = StringOption(
    """ Jinja2 template file for tikz-cd """,
    options='--tikz-cd-template',
    category='html5',
    default='',
)



#!/usr/bin/env python

import datetime
from plasTeX.Tokenizer import Token, EscapeSequence, Other
from plasTeX import Command, Environment, CountCommand
from plasTeX import IgnoreCommand, sourceChildren
from plasTeX.Logging import getLogger

log = getLogger()
status = getLogger('status')
deflog = getLogger('parse.definitions')
envlog = getLogger('parse.environments')
mathshiftlog = getLogger('parse.mathshift')

class relax(Command):
    pass

class protect(Command):
    pass

class global_(Command):
    macroName = 'global'

class par(Command):
    """ Paragraph """
    level = Command.PAR_LEVEL

    def invoke(self, tex):
        status.dot()

    @property
    def source(self):
        if self.hasChildNodes():
            return '%s\n\n' % sourceChildren(self)
        return '\n\n'

    def digest(self, tokens):
        status.dot()

    @property
    def isElementContentWhitespace(self):
        if not self.hasChildNodes():
            return True
        return False

class BoxCommand(Command):
    """ Base class for box-type commands """
    args = 'self'
    mathMode = False
    def parse(self, tex):
        MathShift.inEnv.append(None)
        Command.parse(self, tex)
        MathShift.inEnv.pop()
        return self.attributes

class hbox(BoxCommand): pass
class vbox(BoxCommand): pass

class MathShift(Command):
    """
    The '$' character in TeX

    This macro detects whether this is a '$' or '$$' grouping.  If
    it is the former, a 'math' environment is invoked.  If it is
    the latter, a 'displaymath' environment is invoked.

    """
    macroName = 'active::$'
    inEnv = []

    def invoke(self, tex):
        """
        This gets a bit tricky because we need to keep track of both
        our beginning and ending.  We also have to take into
        account \mbox{}es.

        """
        inEnv = type(self).inEnv

        current = self.ownerDocument.createElement('math')
        for t in tex.itertokens():
            if t.catcode == Token.CC_MATHSHIFT:
                current = self.ownerDocument.createElement('displaymath')
            else:
                tex.pushToken(t)
            break

        # See if this is the end of the environment
        if inEnv and inEnv[-1] is not None and type(inEnv[-1]) is type(current):
            inEnv.pop()
            current.macroMode = Command.MODE_END
            self.ownerDocument.context.pop(current)
            return [current]

        inEnv.append(current)
        mathshiftlog.debug('%s (%s)' % (current.tagName, id(current)))
        self.ownerDocument.context.push(current)

        return [current]

class AlignmentChar(Command):
    """ The '&' character in TeX """
    macroName = 'active::&'

class SuperScript(Command):
    """ The '^' character in TeX """
    macroName = 'active::^'
    args = 'self'
    def invoke(self, tex):
        # If we're not in math mode, just treat this as a normal character
        if not self.ownerDocument.context.isMathMode:
            return tex.textTokens('^')
        Command.parse(self, tex)

class SubScript(Command):
    """ The '_' character in TeX """
    macroName = 'active::_'
    args = 'self'
    def invoke(self, tex):
        # If we're not in math mode, just treat this as a normal character
        if not self.ownerDocument.context.isMathMode:
            return tex.textTokens('_')
        Command.parse(self, tex)

class DefCommand(Command):
    """ TeX's \\def command """
    local = True
    args = 'name:Tok args:Args definition:nox'
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        deflog.debug('def %s %s %s', a['name'], a['args'], a['definition'])
        # See if this definiton has nested parameters
        nested = False
        aiter = iter(a['args'])
        for t in aiter:
            if t.catcode == t.CC_PARAMETER:
                for t in aiter:
                    if t.catcode == t.CC_PARAMETER:
                        nested = True
                    break
            if nested:
                break
        # If we are nested, get rid of one level of #s
        if nested:
            for key in ['args','definition']:
                params = 0
                newarg = []
                for t in a[key]:
                    if t.catcode == t.CC_PARAMETER:
                        params += 1
                        newarg.append(t)
                    else:
                        if params > 1:
                            newarg.pop()
                        newarg.append(t)
                        params = 0
                a[key] = newarg
        self.ownerDocument.context.newdef(a['name'], a['args'], a['definition'], local=self.local)

class def_(DefCommand):
    macroName = 'def'

class edef(DefCommand):
    local = True

class xdef(DefCommand):
    local = False

class gdef(DefCommand):
    local = False

class IfCommand(Command):
    pass

class if_(IfCommand):
    """ \\if """
    args = 'a:Tok b:Tok'
    macroName = 'if'
    """ Test if character codes agree """
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        tex.processIfContent(a['a'] == a['b'])
        return []

class else_(Command):
    macroName = 'else'

class fi(Command):
    pass

class ifnum(IfCommand):
    """ Compare two integers """
    args = 'a:Number rel:Tok'
    def invoke(self, tex):
        self.parse(tex)
        attrs = self.attributes
        attrs['b'] = tex.readNumber(optspace=False)
        relation = attrs['rel']
        a, b = attrs['a'], attrs['b']
        if relation == '<':
            tex.processIfContent(a < b)
            return []
        elif relation == '>':
            tex.processIfContent(a > b)
            return []
        elif relation == '=':
            tex.processIfContent(a == b)
            return []
        raise ValueError('"%s" is not a valid relation' % relation)

class ifdim(IfCommand):
    """ Compare two dimensions """
    args = 'a:Dimen rel:Tok b:Dimen'
    def invoke(self, tex):
        self.parse(tex)
        attrs = self.attributes
        relation = attrs['rel']
        a, b = attrs['a'], attrs['b']
        if relation == '<':
            tex.processIfContent(a < b)
            return []
        elif relation == '>':
            tex.processIfContent(a > b)
            return []
        elif relation == '=':
            tex.processIfContent(a == b)
            return []
        raise ValueError('"%s" is not a valid relation' % relation)

class ifodd(IfCommand):
    """ Test for odd integer """
    def invoke(self, tex):
        tex.processIfContent(not(not(tex.readNumber(optspace=False) % 2)))
        return []

class ifeven(IfCommand):
    """ Test for even integer """
    def invoke(self, tex):
        tex.processIfContent(not(tex.readNumber(optspace=False) % 2))
        return []

class ifvmode(IfCommand):
    """ Test for vertical mode """
    def invoke(self, tex):
        tex.processIfContent(False)
        return []

class ifhmode(IfCommand):
    """ Test for horizontal mode """
    def invoke(self, tex):
        tex.processIfContent(True)
        return []

class ifmmode(IfCommand):
    """ Test for math mode """
    def invoke(self, tex):
        tex.processIfContent(self.ownerDocument.context.isMathMode)
        return []

class ifinner(IfCommand):
    """ Test for internal mode """
    def invoke(self, tex):
        tex.processIfContent(False)
        return []

class ifcat(IfCommand):
    """ Test if category codes agree """
    args = 'a:Tok b:Tok'
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        tex.processIfContent(a['a'].catcode == a['b'].catcode)
        return []

class ifx(IfCommand):
    """ Test if tokens agree """
    args = 'a:XTok b:XTok'
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        tex.processIfContent(a['a'] == a['b'])
        return []

class ifvoid(IfCommand):
    """ Test a box register """
    def invoke(self, tex):
        tex.readNumber(optspace=False)
        tex.processIfContent(False)
        return []

class ifhbox(IfCommand):
    """ Test a box register """
    def invoke(self, tex):
        tex.readNumber(optspace=False)
        tex.processIfContent(False)
        return []

class ifvbox(IfCommand):
    """ Test a box register """
    def invoke(self, tex):
        tex.readNumber(optspace=False)
        tex.processIfContent(False)
        return []

class ifeof(IfCommand):
    """ Test for end of file """
    def invoke(self, tex):
        tex.readNumber(optspace=False)
        tex.processIfContent(False)
        return []

class iftrue(IfCommand):
    """ Always true """
    def invoke(self, tex):
        tex.processIfContent(True)
        return []

class ifplastex(iftrue): pass
class plastexfalse(Command): pass
class plastextrue(Command): pass

class ifhtml(iftrue): pass
class htmlfalse(Command): pass
class htmltrue(Command): pass

class iffalse(IfCommand):
    """ Always false """
    def invoke(self, tex):
        tex.processIfContent(False)
        return []

class ifpdf(iffalse): pass
class pdffalse(Command): pass
class pdftrue(Command): pass
#class pdfoutput(Command): pass

class ifcase(IfCommand):
    """ Cases """
    def invoke(self, tex):
        tex.processIfContent(tex.readNumber(optspace=False))
        return []


class let(Command):
    """ \\let """
    args = 'name:Tok = value:Tok'
    def invoke(self, tex):
        a = self.parse(tex)
        self.ownerDocument.context.let(a['name'], a['value'])

class char(Command):
    """ \\char """
    args = 'char:Number'
    def invoke(self, tex):
        return tex.textTokens(chr(self.parse(tex)['char']))

class chardef(Command):
    args = 'command:cs = num:Number'
    def invoke(self, tex):
        a = self.parse(tex)
        self.ownerDocument.context.chardef(a['command'], a['num'])

class mathchardef(Command):
    args = 'command:cs = num:Number'
    def invoke(self, tex):
        a = self.parse(tex)
        self.ownerDocument.context.mathchardef(a['command'], a['num'])

class NameDef(Command):
    macroName = '@namedef'
    args = 'name:str value:nox'

class makeatletter(Command):
    def invoke(self, tex):
        self.ownerDocument.context.catcode('@', Token.CC_LETTER)

class everypar(Command):
    args = 'tokens:nox'

class catcode(Command):
    """ \\catcode """
    args = 'char:Number = code:Number'
    def invoke(self, tex):
        a = self.parse(tex)
        self.ownerDocument.context.catcode(chr(a['char']), a['code'])
    def source(self):
        return '\\catcode`\%s=%s' % (chr(self.attributes['char']),
                                     self.attributes['code'])
    source = property(source)

class csname(Command):
    """ \\csname """
    def invoke(self, tex):
        name = []
        for t in tex:
            if t.nodeType == Command.ELEMENT_NODE and t.nodeName == 'endcsname':
                break
            name.append(t)
        return [EscapeSequence(''.join(name))]

class endcsname(Command):
    """ \\endcsname """
    pass

class input(Command):
    """ \\input """
    args = 'name:str'
    def invoke(self, tex):
        a = self.parse(tex)
        try:
            path = tex.kpsewhich(a['name'])
            status.info(' ( %s ' % path)
            encoding = self.config['files']['input-encoding']
            with open(path, encoding=encoding) as f:
                tex.input(f.read())
            status.info(' ) ')

        except (OSError, IOError) as msg:
            log.warning(msg)
            status.info(' ) ')

class endinput(Command):
    def invoke(self, tex):
        tex.endInput()

class include(input):
    """ \\include """

class showthe(Command):
    args = 'arg:cs'
    def invoke(self, tex):
        log.info(self.ownerDocument.createElement(self.parse(tex)['arg']).the())


class active(CountCommand):
    value = CountCommand.new(13)

class advance(Command):
    def invoke(self, tex):
        tex.readArgument(type='Number')
        tex.readKeyword(['by'])
        tex.readArgument(type='Number')

class leavevmode(Command): pass

class kern(Command): pass

class hrule(Command): pass

class jobname(Command):
    def invoke(self, tex):
        self.str = tex.jobname

class long(Command): pass

class undefined(Command): pass

class undefined_(Command):
    macroName = '@undefined'

class vobeyspaces_(Command):
    macroName = '@vobeyspaces'

class noligs_(Command):
    macroName = '@noligs'

class expandafter(Command):
    def invoke(self, tex):
        nexttok = None
        for tok in tex.itertokens():
            nextok = tok
            break
        for tok in tex:
            aftertok = tok
            break
        tex.pushToken(aftertok)
        tex.pushToken(nexttok)
        return []

class vskip(Command):
    args = 'size:Dimen'

class hskip(Command):
    args = 'size:Dimen'

class openout(Command):
    args = 'arg:cs = value:any'
    def invoke(self, tex):
        result = Command.invoke(self, tex)
#       a = self.attributes
#       self.ownerDocument.context.newwrite(a['arg'].nodeName,
#                                           a['value'].textContent)
        return result

class closeout(Command):
    args = 'arg:cs'
    def invoke(self, tex):
        result = Command.invoke(self, tex)
#       a = self.attributes
#       self.ownerDocument.context.writes[a['arg'].nodeName].close()
        return result

class write(Command):
    args = 'arg:cs text:nox'
    def invoke(self, tex):
        result = Command.invoke(self, tex)
#       a = self.attributes
#       self.ownerDocument.context.writes[a['arg'].nodeName].write(self.attributes['text']+'\n')
        return result

class protected_write(write):
    nodeName = 'protected@write'

class hfil(Command):
    pass

class the(Command):
    args = 'arg:cs'
    def invoke(self, tex):
        result = Command.invoke(self, tex) 
        name = self.attributes['arg']
        if name == 'year':
            return [Other(datetime.datetime.now().strftime('%Y'))]
        elif name == 'month':
            return [Other(datetime.datetime.now().strftime('%-m'))]
        elif name == 'day':
            return [Other(datetime.datetime.now().strftime('%-d'))]
        return [Other('???')]



#!/usr/bin/env python

"""
C.6.2 List-Making Environments
C.6.3 The list and trivlist Enviroments

"""

from plasTeX import Environment, Command, DimenCommand
from plasTeX import Logging

log = Logging.getLogger()

class enuminame(Command): str = ''
class enumiiname(Command): str = ''
class enumiiiname(Command): str = ''
class enumivname(Command): str = ''

class List(Environment):
    """ Base class for all list-based environments """
    depth = 0
    counters = ['enumi','enumii','enumiii','enumiv']
    blockType = True

    class item(Command):
        args = '[ term ]'
        counter = 'enumi'
        position = 0
        forcePars = True

        def invoke(self, tex):
            """ Set up counter for this list depth """
            try:
                self.counter = List.counters[List.depth-1]
                self.position = self.ownerDocument.context.counters[self.counter].value + 1
            except (KeyError, IndexError):
                pass
            return Command.invoke(self, tex)

        def digest(self, tokens):
            """
            Items should absorb all of the content within that 
            item, not just the `[...]' argument.  This is 
            more useful for the resulting document object.

            """
            for tok in tokens:
                if tok.isElementContentWhitespace:
                    continue
                tokens.push(tok)
                break
            self.digestUntil(tokens, List.item)
            if self.forcePars:
                self.paragraphs()

    def invoke(self, tex):
        """ Set list nesting depth """
        if self.macroMode != Environment.MODE_END:
            List.depth += 1
        else:
            List.depth -= 1
        try:
            for i in range(List.depth, len(List.counters)):
                self.ownerDocument.context.counters[List.counters[i]].setcounter(0)
        except (IndexError, KeyError):
            pass
        return Environment.invoke(self, tex)

    def digest(self, tokens):
        if self.macroMode != Environment.MODE_END:
            # Drop any whitespace before the first item
            for tok in tokens:
                if tok.isElementContentWhitespace:
                    continue
                elif tok.nodeName == 'setcounter':
                    tok.digest([])
                    continue
#               if tok.nodeName != 'item':
#                   log.warning('dropping non-item from beginning of list')
#                   continue
                tokens.push(tok)
                break
        Environment.digest(self, tokens) 

#
# Counters -- enumi, enumii, enumiii, enumiv
#            

# C.6.2
    
class itemize(List): 
    pass

class labelitemi(Command):
    pass
class labelitemii(Command):
    pass
class labelitemiii(Command):
    pass
class labelitemiv(Command):
    pass

class enumerate_(List): 
    macroName = 'enumerate'
    args = '[ type ]'  # Actually defined in the enumerate package, but it doesn't hurt

class description(List): 
    pass

# C.6.3

class trivlist(List): 
    pass

class ConfigurableList(List):
    macroName = 'list'
    args = 'defaultlabel decls:nox'

class topsep(DimenCommand):
    value = DimenCommand.new(0)

class partopsep(DimenCommand):
    value = DimenCommand.new(0)

class itemsep(DimenCommand):
    value = DimenCommand.new(0)

class parsep(DimenCommand):
    value = DimenCommand.new(0)

class leftmargin(DimenCommand):
    value = DimenCommand.new(0)

class rightmargin(DimenCommand):
    value = DimenCommand.new(0)

class listparindent(DimenCommand):
    value = DimenCommand.new(0)

class itemindent(DimenCommand):
    value = DimenCommand.new(0)

class labelsep(DimenCommand):
    value = DimenCommand.new(0)

class labelwidth(DimenCommand):
    value = DimenCommand.new(0)

class makelabel(Command):
    args = 'label'

class usecounter(Command):
    args = 'name:str'



#!/usr/bin/env python

import os, glob
from plasTeX import Command, Environment, TheCounter

def ProcessOptions(options, document):
    context = document.context
    # Lists
    context.newcounter('enumi')
    context.newcounter('enumii', resetby='enumi')
    context.newcounter('enumiii', resetby='enumii')
    context.newcounter('enumiv', resetby='enumiii')

    # Sections
    context.newcounter('part', resetby='volume',
                       format='$part')
    context.newcounter('chapter', resetby='volume',
                       format='$chapter')
    context.newcounter('section', resetby='chapter',
                       format='${thechapter}.${section}')
    context.newcounter('subsection', resetby='section',
                       format='${thesection}.${subsection}')
    context.newcounter('subsubsection', resetby='subsection',
                       format='${thesubsection}.${subsubsection}')
    context.newcounter('paragraph', resetby='subsubsection',
                       format='${thesubsubsection}.${paragraph}')
    context.newcounter('subparagraph', resetby='paragraph',
                       format='${theparagraph}.${subparagraph}')
    context.newcounter('subsubparagraph', resetby='subparagraph',
                       format='${thesubparagraph}.${subsubparagraph}')

    context.newcounter('equation', resetby='chapter',
                       format='${thechapter}.${equation}')

    context.newcounter('secnumdepth')
    context.newcounter('tocdepth')
    context.newcounter('page')

    # Floats
    context.newcounter('figure', resetby='chapter',
                       format='${thechapter}.${figure}')
    context.newcounter('table', resetby='chapter',
                       format='${thechapter}.${table}')
    context.newcounter('topnumber')
    context.newcounter('bottomnumber')
    context.newcounter('totalnumber')
    context.newcounter('dbltopnumber')
    context.loadLanguage('american', document)
    
    language = False
    languages = list(document.context.languages.keys())
    for key, value in list(options.items()):
        if key == 'language':
            language = True
            context.loadLanguage(value, document)
        elif key in languages:
            language = True
            context.loadLanguage(key, document)

class frontmatter(Command):
    pass

class mainmatter(Command):
    pass

class backmatter(Command):
    pass

class appendix(Command):

    class thechapter(TheCounter):
        format = '${chapter.Alph}'

    def invoke(self, tex):
        self.ownerDocument.context.counters['chapter'].setcounter(0)
        self.ownerDocument.context['thechapter'] = type(self).thechapter



#!/usr/bin/env python

from plasTeX.Packages.book import *

class checkandfixthelayout(Command): pass
class setlrmarginsandblock(Command): pass

class titleref(Command):
    args = 'label:idref'

class tightlist(Command):
    def invoke(self, tex):
        return []

class firmlist(Command):
    def invoke(self, tex):
        return []



#!/usr/bin/env python

from collections import UserString
from plasTeX.ConfigManager.Generic import GenericOption, DEFAULTS, GenericParser, GenericArgument
import collections


class StringParser(GenericParser): pass

class StringOption(StringParser, GenericOption, UserString):
   """ String configuration option """

   synopsis = ''

   def __init__(self, docstring=DEFAULTS['docstring'],
                      options=DEFAULTS['options'],
                      default=DEFAULTS['default'],
                      optional=DEFAULTS['optional'],
                      values=DEFAULTS['values'],
                      category=DEFAULTS['category'],
                      callback=DEFAULTS['callback'],
                      synopsis=DEFAULTS['synopsis'],
                      environ=DEFAULTS['environ'],
                      registry=DEFAULTS['registry'],
                      mandatory=None,
                      name=DEFAULTS['name'],
                      source=DEFAULTS['source']):
      UserString.__init__(self, '')
      GenericOption.initialize(self, locals())
      self.data = self.data or ''

   def cast(self, arg):
      if arg is None: return
      return str(arg)

   def __iadd__(self, other):
      if isinstance(self.callback, collections.Callable):
         other = self.callback(self.cast(other))

      if other is None:
         return self

      if self.data is None:
         self.data = self.cast(other)
      else:
         self.data += '\n%s' % self.cast(other)

      return self


class StringArgument(GenericArgument, StringOption):
   """ String command-line option """



import os
import tempfile
from jinja2 import Template
from bs4 import BeautifulSoup

from plasTeX import NoCharSubEnvironment, Command
from plasTeX.PackageResource import PackageResource
from plasTeX.Packages import tikz

from plasTeX.Logging import getLogger
log = getLogger()

class tikzcd(NoCharSubEnvironment):
    """
    A tikz-cd diagram whose content will be converted in the processFileContent callback.
    """


    class ar(Command):
        pass

    class rar(Command):
        pass

    class lar(Command):
        pass

    class uar(Command):
        pass

    class drar(Command):
        pass

    class dar(Command):
        pass

    class dlar(Command):
        pass

    class ular(Command):
        pass

    class urar(Command):
        pass


def ProcessOptions(options, document):
    """This is called when the package is loaded."""

    try:
        with open(document.config['html5']['tikz-cd-template'], "r") as file:
            template = file.read()
    except IOError:
        log.info('Using default TikZ template.')
        template = u"\\documentclass{standalone}\n\\usepackage{tikz-cd}" + \
                   u"\\begin{document}{{ tikzpicture }}\\end{document}"
    document.userdata['tikzcd'] = {
            'template': Template(template),
            'tmp_dir': tempfile.mkdtemp(),
            'compiler': document.config['html5']['tikz-compiler'],
            'pdf2svg': document.config['html5']['tikz-converter'],
            }

    def convert(document, content):
        return tikz.tikzConvert(
                document,
                content,
                'tikzcd',
                'Commutative diagram')

    cb = PackageResource(
            renderers='html5',
            key='processFileContents',
            data=convert)
    document.addPackageResource(cb)

    cb = PackageResource(
            renderers='gerby',
            key='processFileContents',
            data=convert)
    document.addPackageResource(cb)



#!/usr/bin/env python

from plasTeX import Command
from plasTeX.Base.LaTeX import Accents

class textonequarter(Command):
    str = '\u00BC'

class textonehalf(Command):
    str = '\u00BD'

class textthreequarters(Command):
    str = '\u00BE'

class textzerooldstyle(Command):
    str = '0'

class textoneoldstyle(Command):
    str = '1'

class texttwooldstyle(Command):
    str = '2'

class textthreeoldstyle(Command):
    str = '3'

class textfouroldstyle(Command):
    str = '4'

class textfiveoldstyle(Command):
    str = '5'

class textsixoldstyle(Command):
    str = '6'

class textsevenoldstyle(Command):
    str = '7'

class texteightoldstyle(Command):
    str = '8'

class textnineoldstyle(Command):
    str = '9'

class textflorin(Command):
    str = '\u0192'

class textyen(Command):
    str = '\u00a5'

class textwon(Command):
    str = '\u20a9'

class textnaira(Command):
    str = '\u20a6'

class textpeso(Command):
    str = '\u20b1'

class textborn(Command):
    str = '\u2605'

class textdied(Command):
    str = '\u2020'

class textmarried(Command):
    str = '\u26ad'

class textdivorced(Command):
    str = '\u26ae'

class textcelsius(Command):
    str = '\u00b0C'

class textopenbullet(Command):
    str = '\u25e6'

class textdollaroldstyle(Command):
    str = '$'

class textsterling(Command):
    str = '\u00a3'

class textlira(Command):
    str = '\u20a4'

class textguarani(Command):
    str = '\u20b2'

class texteuro(Command):
    str = '\u20ac'

class textcentoldstyle(Command):
    str = '\u00a2'

class textestimated(Command):
    str = '\u212e'

class newtie(Accents.t):
    macroName = None

class capitaltie(Accents.t):
    macroName = None

class capitalacute(Accents.Acute):
    macroName = None

class capitaldieresis(Accents.Umlaut):
    macroName = None

class capitalbreve(Accents.Macron):
    macroName = None

class capitalnewtie(Accents.t):
    macroName = None

class capitalgrave(Accents.Grave):
    macroName = None

class capitaldotaccent(Accents.Dot):
    macroName = None

class capitalcedilla(Accents.c):
    macroName = None



#!/usr/bin/env python

"""
This is a huge package!  Most of the commands are simply stubbed out
and not implemented yet.  Of course, many of the commands simply won't
need anything more since they either don't make sense for plasTeX or
the renderer will be doing all the work.

"""

from plasTeX import Command

def ProcessOptions(options, document):
    context = document.context
    context.newcounter('partlofdepth')
    context.newcounter('partlotdepth')
    context.newcounter('parttocdepth')
    context.newcounter('minilofdepth')
    context.newcounter('minilotdepth')
    context.newcounter('minitocdepth')
    context.newcounter('seclofdepth')
    context.newcounter('seclotdepth')
    context.newcounter('sectocdepth')
    context.newcounter('mtc')
    context.newcounter('ptc')
    context.newcounter('stc')

class dominitoc(Command):
    args = '[ pos:str ]'

class dominilof(dominitoc):
    pass

class dominilot(dominitoc):
    pass

class _minitoccommand(Command):
    args = '[ pos:str ]'
    titleMacro = ''

    def invoke(self, tex):
        Command.invoke(self, tex)
        self.title = self.ownerDocument.createElement(self.titleMacro).expand(tex)

    @property
    def tableofcontents(self):
        node = self.parentNode
        while node is not None and node.level > Command.ENDSECTIONS_LEVEL:
            node = node.parentNode
        if node is not None:
            return node.tableofcontents

class minitoc(_minitoccommand):
    titleMacro = 'mtctitle'

class minilof(_minitoccommand):
    titleMacro = 'mlftitle'

class minilot(_minitoccommand):
    titleMacro = 'mlttitle'

class dosecttoc(dominitoc):
    pass

class dosectlof(dominitoc):
    pass

class dosectlot(dominitoc):
    pass

class secttoc(minitoc):
    titleMacro = 'stctitle'

class sectlof(minilof):
    titleMacro = 'slftitle'

class sectlot(minilot):
    titleMacro = 'slttitle'

class doparttoc(dominitoc):
    pass

class dopartlof(dominilof):
    pass

class dopartlot(dominilot):
    pass

class parttoc(minitoc):
    titleMacro = 'ptctitle'

class partlof(minilof):
    titleMacro = 'plftitle'

class partlot(minilot):
    titleMacro = 'plttitle'

# 
# Formatting
#

class mtcsetfont(Command):
    args = 'which:str level:str format:nox'

class mtcsetdepth(Command):
    args = 'which:str depth:int'

class mtcprepare(Command):
    args = 'toc:str level:str commands'

class mtcskip(Command):
    pass

class mtcskipammount(Command):
    pass

class tightmtcfalse(Command):
    pass

class tightmtctrue(Command):
    pass

class ktightmtcfalse(Command):
    pass

class ktightmtctrue(Command):
    pass

class undottedmtcfalse(Command):
    pass

class undottedmtctrue(Command):
    pass

class addstarredpart(Command):
    args = 'title'

class addstarredchapter(Command):
    args = 'title'

class addstarredsection(Command):
    args = 'title'

class _adjust(Command):
    args = '[ num:int ]'
    default = 1
    counterName = ''
    def invoke(self, tex):
        Command.invoke(self, tex)
        n = self.attributes['num'] or self.default
        self.ownerDocument.context.counters[self.counterName].addtocounter(n)

class adjustptc(_adjust):
    counterName = 'ptc'

class adjustmtc(_adjust):
    counterName = 'mtc'

class adjuststc(_adjust):
    counterName = 'stc'

class _decrement(Command):
    def invoke(self, tex):
        Command.invoke(self, tex)
        self.ownerDocument.context.counters[self.counterName].addtocounter(-1)

class decrementptc(_decrement):
    counterName = 'ptc'

class decrementmtc(_decrement):
    counterName = 'mtc'

class decrementstc(_decrement):
    counterName = 'stc'

class _increment(Command):
    def invoke(self, tex):
        Command.invoke(self, tex)
        self.ownerDocument.context.counters[self.counterName].addtocounter(1)

class incrementptc(_increment):
    counterName = 'ptc'

class incrementmtc(_increment):
    counterName = 'mtc'

class incrementstc(_increment):
    counterName = 'stc'

class mtcaddpart(Command):
    args = '[ title ]'

class mtcaddchapter(Command):
    args = '[ title ]'

class mtcaddsection(Command):
    args = '[ title ]'

#
# Titles
#

class mtctitle(Command):
    str = 'Contents'

class mlftitle(Command):
    str = 'Figures'

class mlttitle(Command):
    str = 'Tables'

class ptctitle(Command):
    str = 'Table of Contents'

class plftitle(Command):
    str = 'List of Figures'

class plttitle(Command):
    str = 'List of Tables'

class stctitle(Command):
    str = 'Contents'

class slftitle(Command):
    str = 'Figures'

class slttitle(Command):
    str = 'Tables'

class mtcsettitle(Command):
    args = 'which:str title'

class mtcsettitlefont(Command):
    args = 'which:str commands:nox'

class mtcsetformat(Command):
    args = 'which:str param:str value:nox'

#
# Rules
#

class mtcsetrules(Command):
    args = 'which:str value'

class ptcrule(Command):
    pass

class noptcrule(Command):
    pass

class mtcrule(Command):
    pass

class nomtcrule(Command):
    pass

class stcrule(Command):
    pass

class nostcrule(Command):
    pass

class plfrule(Command):
    pass

class noplfrule(Command):
    pass

class mlfrule(Command):
    pass

class nomlfrule(Command):
    pass

class slfrule(Command):
    pass

class noslfrule(Command):
    pass

class pltrule(Command):
    pass

class nopltrule(Command):
    pass

class mltrule(Command):
    pass

class nomltrule(Command):
    pass

class sltrule(Command):
    pass

class nosltrule(Command):
    pass


# 
# Page Numbers
#

class mtcsetpagenumbers(Command):
    args = 'which:str value'

class ptcpagenumbers(Command):
    pass

class noptcpagenumbers(Command):
    pass

class plfpagenumbers(Command):
    pass

class noplfpagenumbers(Command):
    pass

class pltpagenumbers(Command):
    pass

class nopltpagenumbers(Command):
    pass

class mtcpagenumbers(Command):
    pass

class nomtcpagenumbers(Command):
    pass

class mlfpagenumbers(Command):
    pass

class nomlfpagenumbers(Command):
    pass

class mltpagenumbers(Command):
    pass

class nomltpagenumbers(Command):
    pass

class stcpagenumbers(Command):
    pass

class nostcpagenumbers(Command):
    pass

class slfpagenumbers(Command):
    pass

class noslfpagenumbers(Command):
    pass

class sltpagenumbers(Command):
    pass

class nosltpagenumbers(Command):
    pass


class mtcsetfeature(Command):
    args = 'which:str param:str commands:nox'

class firstpartis(Command):
    args = 'num:int'

class firstchapteris(Command):
    args = 'num:int'

class firstsectionis(Command):
    args = 'num:int'

class mtcfixglossary(Command):
    pass

class mtcfixindex(Command):
    pass

class faketableofcontents(Command):
    pass

class fakelistoffigures(Command):
    pass

class fakelistoftables(Command):
    pass

class mtcselectlanguage(Command):
    args = 'language:str'





#!/usr/bin/env python

"""
ConfigManager

ConfigManager is a combination command-line option parser and configuration
file.  It essentially combines ConfigParser, getopt, and a lot of
additional logic to parse the command-line the way you expect it to be
parsed.  The ConfigManager class should be backwards compatible with the
ConfigParser class, but contains much more functionality and a more natural
dictionary-style interface to sections and options.

See examples at the bottom of this file.  Try typing __init__.py
followed by a bunch of imaginary command line options and arguments.

"""

import sys, string, re, urllib.request, copy, types, os
from collections import UserList, UserDict, OrderedDict
from textwrap import wrap

__all__ = ['ConfigManager','BooleanOption','IntegerOption','CompoundOption',
           'MultiOption','GenericOption','FloatOption','StringOption',
           'InputFileOption','OutputFileOption','InputDirectoryOption',
           'OutputDirectoryOption','CountedOption',
           'BooleanArgument','IntegerArgument','CompoundArgument',
           'MultiArgument','GenericArgument','FloatArgument','StringArgument',
           'InputFileArgument','OutputFileArgument','InputDirectoryArgument',
           'OutputDirectoryArgument','CountedArgument',
           'BUILTIN','CODE','REGISTRY','CONFIG','CONFIGFILE','ENVIRON',
           'ENVIRONMENT','COMMANDLINE','ALL','DEFAULTSECT',
           'ON','OFF','TRUE','FALSE','YES','NO','CommandLineManager',
           'GetoptError','ConfigError','NoOptionError']

DEFAULTSECT = "DEFAULT"
MAX_INTERPOLATION_DEPTH = 10
ON = TRUE = YES = 1
OFF = FALSE = NO = 0

TERMINAL_WIDTH = 76   # Maximum width of terminal
MAX_NAME_WIDTH_RATIO = 0.25  # Max fraction of terminal to use for option
PREPAD = 2   # Padding before each option name in usage
GUTTER = 4   # Space between option name and description in usage

# Possible values for `source'.
BUILTIN = 2
CODE = 4
REGISTRY = 8
CONFIG = CONFIGFILE = 16
ENVIRON = ENVIRONMENT = 32
COMMANDLINE = 64
ALL = 0xffffff

# Exception classes
class Error(Exception):
    """ Generic exception """
    def __init__(self, msg=''):
        self.msg = msg
        Exception.__init__(self, msg)

    def __str__(self):
        return self.msg
    __repr__ = __str__


# Exceptions while parsing command line
class GetoptError(Error):
    """ Generic command line exception """
    def __init__(self, msg, opt):
        self.msg = msg
        self.opt = opt
        Exception.__init__(self, msg, opt)

    def __str__(self):
        return self.msg
    __repr__ = __str__

class RequiresArgument(GetoptError):
    """ Exception for a missing argument """

class MandatoryOption(GetoptError):
    """ Exception for a missing option """

class UnspecifiedArgument(GetoptError):
    """ Exception for an argument when none was expected """

class UnrecognizedArgument(GetoptError):
    """ Exception for an argument that is unrecognized """

class NonUniquePrefix(GetoptError):
    """ Exception for multiple option prefixes that match a given option """

class UnknownCompoundGroup(GetoptError):
    """ Exception for an unknown grouping character used for a compound """
    def __init__(self, msg=''):
        GetoptError.__init__(self, msg, '')


# Exceptions while reading/parsing configuration files
class ConfigError(Error):
   """ Generic configuration file exception """

class NoSectionError(ConfigError):
    """ Exception for missing sections """
    def __init__(self, section):
        ConfigError.__init__(self, 'No section: %s' % section)
        self.section = section

class DuplicateSectionError(ConfigError):
    """ Exception for duplicate sections """
    def __init__(self, section):
        ConfigError.__init__(self, "Section %s already exists" % section)
        self.section = section

class InvalidOptionError(GetoptError, ConfigError):
    """ Exception for invalid values for a given option """
    def __init__(self, option, value, msg='', type=''):
        if type: type += ' '
        if not msg:
            msg="Invalid value for %soption `%s'" % (type, option)
        ConfigError.__init__(self, msg+': %s' % value)
        self.option = option
        self.value = value

class NoOptionError(ConfigError):
    """ Exception for missing a missing option in a section """
    def __init__(self, option, section):
        ConfigError.__init__(self, "No option `%s' in section: %s" %
                       (option, section))
        self.option = option
        self.section = section

class InterpolationError(ConfigError):
    """ Exception for message interpolation errors """
    def __init__(self, reference, option, section, rawval):
        ConfigError.__init__(self,
                       "Bad value substitution:\n"
                       "\tsection: [%s]\n"
                       "\toption : %s\n"
                       "\tkey    : %s\n"
                       "\trawval : %s\n"
                       % (section, option, reference, rawval))
        self.reference = reference
        self.option = option
        self.section = section

class InterpolationDepthError(ConfigError):
    """ Exception for excessive recursion in interpolation """
    def __init__(self, option, section, rawval):
        ConfigError.__init__(self,
                       "Value interpolation too deeply recursive:\n"
                       "\tsection: [%s]\n"
                       "\toption : %s\n"
                       "\trawval : %s\n"
                       % (section, option, rawval))
        self.option = option
        self.section = section

class ParsingError(ConfigError):
    """ Exception for errors occurring during parsing of a config file """
    def __init__(self, filename):
        ConfigError.__init__(self, 'File contains parsing errors: %s' % filename)
        self.filename = filename
        self.errors = []

    def append(self, lineno, line):
        self.errors.append((lineno, line))
        self.msg = self.msg + '\n\t[line %2d]: %s' % (lineno, line)

class TooFewValues(GetoptError):
   """ Got fewer values than expected """
   def __init__(self, msg):
      GetoptError.__init__(self, msg, '')

class TooManyValues(GetoptError):
   """ Got more values than expected """
   def __init__(self, msg):
      GetoptError.__init__(self, msg, '')

class MissingSectionHeaderError(ParsingError):
    """ Exception for options that occur before a section heading """
    def __init__(self, filename, lineno, line):
        ConfigError.__init__(
            self,
            'File contains no section headers.\nfile: %s, line: %d\n%s' %
            (filename, lineno, line))
        self.filename = filename
        self.lineno = lineno
        self.line = line


class ConfigSection(UserDict, object):
    """ Section of a configuration object """

    def __init__(self, name, data=None):
        """
        Initialize the section

        Required Arguments:
        name -- name of the section
        data -- dictionary containing the initial set of options

        """
        UserDict.__init__(self, data or {})
        self.name = name
        self.parent = None

    def copy(self):
        """ Make a deep copy of self """
        newcopy = self.__class__(self.name)
        for key, value in list(vars(self).items()):
            if key == 'data': continue
            setattr(newcopy, key, value)
        for key, value in list(self.data.items()):
            newcopy.data[key] = value.copy()
        return newcopy

    def setParent(self, parent):
        """ Set the parent ConfigManager instance """
        self.parent = parent

    def defaults(self):
        """ Return the dictionary of defaults """
        return self.parent.defaults()

    def __getitem__(self, key):
        """ Return the value of the option, not the option itself """
        return self.get(key)

    def set(self, option, value, source=BUILTIN):
        """
        Create the appropriate option type

        If the value is already an Option instance, just plug it in.
        If the value is anything else, try to figure out which option
        type it corresponds to and create an option of that type.

        Required Arguments:
        option -- dictionary key where the option should be set
        value -- option value to store
        source -- flag to indicate source of option

        Returns: None

        """
        typemap = {str:StringOption, int:IntegerOption,
                   float:FloatOption, list:MultiOption, tuple:MultiOption}

        if option in list(self.data.keys()):
           if self.data[option].source <= source:
              self.data[option].source = source
              self.data[option].setValue(value)

        else:
           if isinstance(value, GenericOption):
              value.setParent(self)
              value.name = str(option)
              self.data[option] = value

           elif type(value) in list(typemap.keys()):
              for key, opttype in list(typemap.items()):
                 if isinstance(value, key):
                    # Handle booleans this way until support for
                    # true booleans shows up in Python.
                    if type(value) == str and \
                       value.lower().strip() in ['on','off','true','false','yes','no']:
                       opttype = BooleanOption
                    self.data[option] = opttype(name=option, source=source)
                    self.data[option].setParent(self)
                    self.data[option].name = str(option)
                    self.data[option].setValue(value)
                    break
           else:
              raise TypeError('Could not find valid option type for "%s"' % value)

    def __setitem__(self, key, value):
        """ Set the item in the dictionary """
        self.set(key, value, source=BUILTIN)

    def getint(self, option):
        """ Get the option value and cast it to an integer """
        return int(self[option])

    def getfloat(self, option):
        """ Get the option value and cast it to a float """
        return float(self[option])

    def getboolean(self, option):
        """ Get the option value and cast it to a boolean """
        v = self[option]
        val = int(v)
        if val not in (0, 1):
            raise ValueError('Not a boolean: %s' % v)
        return val

    def get(self, option, raw=0, vars=None):
        """
        Get an option value for a given section.

        All % interpolations are expanded in the return values, based on the
        defaults passed into the constructor, unless the optional argument
        `raw' is true.  Additional substitutions may be provided using the
        `vars' argument, which must be a dictionary whose contents overrides
        any pre-existing defaults.

        Required Arguments:
        option -- name of the option to retrieve

        Keyword Arguments:
        raw -- boolean flag that indicates whether string values should
           be returned as a raw value or as a string with all variable
           interpolation applied
        vars -- dictionary of values to use in string interpolation

        Returns:
        value of the option

        """
        vars = vars or {}
        value = self.getraw(option, vars)

        # Raw was specified
        if raw or value == None: return value

        # If we have a list, see if any strings need interpolation.
        if type(value) in [list, tuple]:
            strings = [s for s in value
                         if isinstance(s,str) and s.find('%(')+1]
            if not strings: return value

        # If we have a string, but no interpolation is needed, bail out.
        elif not(isinstance(value,str)) or value.find("%(") < 0:
            return value

        # otherwise needs interpolation...
        var_dict = self.defaults().data.copy()
        var_dict.update(self.data)
        var_dict.update(vars)

        # Handle lists of interpolations as well as single values.
        if type(value) in [list, tuple]:
            new_values = []
            for i in value:
                new_values.append(self.interpolate(option, var_dict, i))
            return new_values
        else:
            return self.interpolate(option, var_dict, value)

    def interpolate(self, option, vars, rawval):
        """
        Do the string interpolation

        Required Arguments:
        option -- name of the option
        vars -- dictionary of values to use in interpolation
        rawval -- raw value of the option

        Returns:
        string -- string with all variables interpolated

        """
        value = rawval
        depth = 0
        # Loop through this until it's done
        while depth < MAX_INTERPOLATION_DEPTH:
            depth = depth + 1
            if value.find("%(") >= 0:
                try:
                    value = value % vars
                except KeyError as key:
                    raise InterpolationError(key, option, self.name, rawval)
            else:
                break
        if value.find("%(") >= 0:
            raise InterpolationDepthError(option, self.name, rawval)
        return value

    def getraw(self, option, vars=None):
        """
        Return raw value of option

        Required Arguments:
        option -- name of the option to retrieve

        Keyword Arguments:
        vars -- dictionary containing additional default values

        """
        vars = vars or {}
        if option in list(vars.keys()):
            return vars[option].getValue()

        if option in list(self.keys()):
            return self.data[option].getValue()

        defaults = self.defaults()
        if option in list(defaults.keys()):
            return defaults.data[option].getValue()

        raise NoOptionError(option, self.name)

    def to_string(self, source=COMMANDLINE):
        """
        Convert the section back into INI format

        Keyword Arguments:
        source -- flag which indicates which source of information to print

        Returns:
        string -- string containing the section in INI format

        """
        s = ''
        keys = list(self.keys())
        keys.sort()
        for key in keys:
            if source & self.data[key].source:
               raw = self.getraw(key)
               option = self.data[key]

               # Bypass unset options
               if isinstance(option, MultiOption) and raw == []: continue
               if raw == None: continue

               # Print description or summary of the option as well
               comment = ''
               if option.summary: comment = option.summary
               if option.description: comment = option.description
               if comment:
                  comment = comment.strip() % option.names()
                  comment = comment.split('\n')
                  s += '\n; %s\n' % '\n; '.join(comment)

               value = str(option).replace('\n', '\n    ')
               if value.find('\n') + 1: value = '\n    ' + value
               s += "%s %s %s\n" % (key, ConfigManager.OPTIONSEP, value)
        return s

    def __str__(self):
        """ Return section in INI format without builtins """
        return self.to_string()

    def __repr__(self):
        """ Return section in INI format with builtins """
        return self.to_string(ALL)


class ConfigManager(UserDict, object):

    # Regular expressions for parsing section headers and options.
    SECTCRE = re.compile(
        r'\['                                 # [
        r'(?P<header>[^]]+)'                  # very permissive!
        r'\]'                                 # ]
        )

    OPTCRE = re.compile(
        r'(?P<option>[]\-[\w_.*,(){}]+)'      # a lot of stuff found by IvL
        r'[ \t]*(?P<vi>[:=])[ \t]*'           # any number of space/tab,
                                              # followed by separator
                                              # (either : or =), followed
                                              # by any # space/tab
        r'(?P<value>.*)$'                     # everything up to eol
        )

    # Option separator used in printing out INI format
    OPTIONSEP = '='

    # Set prefixes for options.  If these are the same, all options
    # are treated as long options.  You can set either one to None
    # to turn that type of option off as well.
    short_prefix = '-'
    long_prefix = '--'

    def __init__(self, defaults=None):
        """
        Initialize ConfigManager

        Keyword Arguments:
        defaults -- dictionary of default values.  These values will
           make up the section by the name DEFAULTSECT.

        """
        defaults = defaults or {}
        UserDict.__init__(self)
        self[DEFAULTSECT] = ConfigSection(DEFAULTSECT, defaults)
        self.strict = 1     # Raise exception for unknown options
        self._categories = {}  # Dictionary of option categories
        self.unrecognized = []

    def copy(self):
        """ Make a deep copy of self """
        newcopy = self.__class__()
        for key, value in list(vars(self).items()):
            if key == 'data': continue
            setattr(newcopy, key, value)
        for key, value in list(self.data.items()):
            newcopy.data[key] = value.copy()
        return newcopy

    def set_prefixes(cls, arg1, arg2=None):
        """
        Set the command-line option prefixes

        Arguments:
        short - prefix to use for short command-line options.  If this
           is set to 'None', then all options are treated as long options.
        long - prefix to use for long options

        """
        if arg1 == arg2 == None:
           raise ValueError('Short and long prefixes cannot both be None.')
        if arg2 is None:
           cls.long_prefix = arg1
           cls.short_prefix = None
        else:
           cls.long_prefix = arg2
           cls.short_prefix = arg1

    set_prefixes = classmethod(set_prefixes)

    def add_help_on_option(self, category=None):
        """
        Add a --help-on=LIST option for displaying specific option help

        The --help-on= option can be added to give the command line
        interactive help.  For example, if you had an option called
        '--debug', you could type '--help-on debug' to get the full
        description of the --debug option printed out.

        Keyword Arguments:
        category -- category to put the --help-on option in

        """
        self[DEFAULTSECT]['__help_on__'] = MultiOption(
           """ Display help on listed option names """,
           options = '%shelp-on' % self.long_prefix[0],
           category = category,
           callback = self.usage_on,
        )

    def remove_help_on_option(self):
        """ Remove the --help-on option """
        try: del self[DEFAULTSECT]['__help_on__']
        except: pass

    def add_category(self, key, title):
        """
        Add a category to the ConfigManager

        Options can be grouped by categories for display in the usage
        message.  Categories have both a key and a title.  The key is
        what is used in the 'category' parameter when instantiating an
        option.  The title is the heading to print in the usage message.

        Required Arguments:
        key -- name of the category used in instantiating an option
        title -- title of the category to print in the usage message

        Returns:
        string -- the same key given as the first argument

        """
#       if not self._categories:
#          self._categories['__categories__'] = 'Help Categories'
#       if not self.has_key('__categories__'):
#          self.add_section('__categories__')
#       self['__categories__'][key] = BooleanOption(title,
#            options='%shelp-%s' % (self.long_prefix, key),
#            category='__categories__')
        self._categories[key] = title
        return key

    def get_category(self, key):
        """ Return the title of the given category """
        if type(key) not in [list, tuple]:
           key = [key]
        if not key:
           return ''
        return self._categories[key[0]]

    def categories(self):
        """ Return the dictionary of categories """
        return self._categories

    def set_strict(self, bool=1):
        """
        Parse the command line strictly

        If you do not want to be bothered with errors due to unrecognized
        arguments, this method can be called with a boolean 'false'.
        This is very useful if your program is actually a wrapper around
        another program and you do not want to declare all of its options
        in your ConfigManager.  The ConfigManager will simply make its best
        guess as to whether the option accepts a value and what type the
        option is.

        Keyword Arguments:
        bool -- flag indicating whether parsing should be strict or not

        """
        self.strict = not(not(bool))

    def defaults(self):
        """ Return a dictionary of defaults """
        return self[DEFAULTSECT]

    def sections(self):
        """ Return a list of section names """
        return list(self.keys())

    def add_section(self, section):
        """
        Create a new section in the configuration.

        Do nothing if a section by the specified name already exists.

        Required Arguments:
        section -- name of the section to create

        Returns:
        instance -- a ConfigSection instance with the given name is returned

        """
        if section in list(self.keys()): return self[section]
        self[section] = ConfigSection(section)
        return self[section]

    def has_section(self, section):
        """ Indicate whether the named section is present """
        return section in list(self.keys())

    def options(self, section):
        """ Return a list of option names for the given section name """
        if section in list(self.keys()):
            return list(self[section].keys())
        else:
            raise NoSectionError(section)

    def read(self, filenames):
        """
        Read and parse a filename or a list of filenames

        Files that cannot be opened are silently ignored; this is
        designed so that you can specify a list of potential
        configuration file locations (e.g. current directory, user's
        home directory, systemwide directory), and all existing
        configuration files in the list will be read.  A single
        filename may also be given.

        """
        if isinstance(filenames, str):
            filenames = [filenames]
        for filename in filenames:
            try:
                if filename.startswith('~'):
                    filename = os.path.expanduser(filename)

                fp = open(filename)
            except (OSError, IOError):
                continue
            self.__read(fp, filename)
            fp.close()
        return self

    def readfp(self, fp, filename=None):
        """
        Like read() but the argument must be a file-like object.

        The 'fp' argument must have a 'readline' method.  Optional
        second argument is the 'filename', which if not given, is
        taken from fp.name.  If fp has no 'name' attribute, '<???>' is
        used.

        Required Arguments:
        fp -- file-type like object
        filename -- name of the file in 'fp'

        Returns:
        string -- contents of the file pointer

        """
        if filename is None:
            try:
                filename = fp.name
            except AttributeError:
                filename = '<???>'
        self.__read(fp, filename)
        return self

    def get(self, section, option, raw=0, vars=None):
        """ Get an option value for a given section """
        return self[section].get(option, raw, vars or {})

    def set(self, section, option, value, source=BUILTIN):
        """ Set an option value """
        if not section or section == DEFAULTSECT:
            sectdict = self[DEFAULTSECT]
        else:
            try:
                sectdict = self[section]
            except KeyError:
                raise NoSectionError(section)
        sectdict.set(option, value, source)

    def __setitem__(self, key, value):
        """ Add a section to the configuration """
        if isinstance(value, ConfigSection):
           self.data[key] = value
           self.data[key].setParent(self)
        else:
           self.data[key] = ConfigSection(str(key))
           self.data[key].setParent(self)

    def __getitem__(self, key):
        """
        Return section with given name

        Return the section or if it's not a section try to
        return an option by that name in the 'default' section.

        """
        if key in list(self.data.keys()):
           return self.data[key]
        if key in list(self.data[DEFAULTSECT].keys()):
           return self.data[DEFAULTSECT][key]
        raise NoSectionError(key)

    def getint(self, section, option):
        """ Get an option value and cast it to an integer """
        return self[section].getint(option)

    def getfloat(self, section, option):
        """ Get an option value and cast it to a float """
        return self[section].getfloat(option)

    def getboolean(self, section, option):
        """ Get an option value and cast it to a boolean """
        return self[section].get(option)

    def getraw(self, section, option):
        """ Get the raw (un-interpolated) value of an option """
        return self[section].getraw(option)

    def has_option(self, section, option):
        """ Check for the existence of a given option in a given section """
        if not section:
            section=DEFAULTSECT
        elif section not in list(self.keys()):
            return 0
        else:
            return option in list(self[section].keys())

    def write(self, fp):
        """ Write an INI-format representation of the configuration state """
        fp.write(str(self))

    def __str__(self):
        """ Return an INI formated string with builtins removed """
        return self.to_string()

    def __repr__(self):
        """ Return an INI formated string with builtins included """
        return self.to_string(source=COMMANDLINE|CONFIGFILE|CODE|BUILTIN|REGISTRY|ENVIRONMENT)

    def to_string(self, source=COMMANDLINE|CONFIGFILE):
        """
        Build an INI formatted string based on the ConfigManager contents

        Keyword Arguments:
        source -- flag indicating which sources if information to print

        Returns:
        string -- INI formatted string

        """
        if source & BUILTIN: func = repr
        else: func = str
        s = ''
        keys = [x for x in list(self.keys()) if x != DEFAULTSECT]
        keys.sort()
        if self[DEFAULTSECT]:
            keys.insert(0, DEFAULTSECT)
        for section in keys:
            content = func(self[section]).strip()
            if content:
               s += "[%s]\n%s\n\n" % (section, content)
        return s

    def remove_option(self, section, option):
        """ Remove an option from a section """
        if not section or section == DEFAULTSECT:
            sectdict = self[DEFAULTSECT]
        else:
            try:
                sectdict = self[section]
            except KeyError:
                raise NoSectionError(section)
        try:
            del sectdict[option]
            return 1
        except KeyError:
            return 0

    def remove_section(self, section):
        """ Remove the given section """
        if section in list(self.keys()):
            del self[section]
            return 1
        else:
            return 0

    def __read(self, fp, fpname):
        """
        Parse an INI formatted file

        The sections in the file contain a title line at the top,
        indicated by a name in square brackets (`[]'), plus key/value
        options lines, indicated by `name: value' format lines.
        Continuation are represented by an embedded newline then
        leading whitespace.  Blank lines, lines beginning with a '#',
        and just about everything else is ignored.

        """
        cursect = None                            # None, or a dictionary
        optname = None
        lineno = 0
        e = None                                  # None, or an exception
        while 1:
            line = fp.readline()
            if not line:
                break
            lineno = lineno + 1
            # comment or blank line?
            if line.strip() == '' or line[0] in '#;':
                continue
            if line.split()[0].lower() == 'rem' \
               and line[0] in "rR":      # no leading whitespace
                continue
            # continuation line?
            if line[0] in ' \t' and cursect is not None and optname:
                value = line.strip()
                if value and cursect.data[optname].source == CONFIGFILE:
                    cursect.data[optname] += "%s" % value
            # a section header or option header?
            else:
                # is it a section header?
                mo = self.SECTCRE.match(line)
                if mo:
                    sectname = mo.group('header')
                    if sectname in list(self.keys()):
                        cursect = self[sectname]
                    else:
                        cursect = ConfigSection(sectname)
                        self[sectname] = cursect
                    # So sections can't start with a continuation line
                    optname = None
                # no section header in the file?
                elif cursect is None:
                    raise MissingSectionHeaderError(fpname, lineno, repr(line))
                # an option line?
                else:
                    mo = self.OPTCRE.match(line)
                    if mo:
                        optname, vi, optval = mo.group('option', 'vi', 'value')
                        if vi in ('=', ':') and ';' in optval:
                            # ';' is a comment delimiter only if it follows
                            # a spacing character
                            pos = optval.find(';')
                            if pos and optval[pos-1] in string.whitespace:
                                optval = optval[:pos]
                        optval = optval.strip()
                        # allow empty values
                        if optval == '""':
                            optval = ''
                        try:
                            cursect.set(optname, optval, source=CONFIGFILE)
                            cursect.data[optname].file = fpname
                        except:
                            print("Problem occurred in section '%s' while reading file %s." % (cursect.name, fpname))
                            raise
                    else:
                        # a non-fatal parsing error occurred.  set up the
                        # exception but keep going. the exception will be
                        # raised at the end of the file and will contain a
                        # list of all bogus lines
                        if not e:
                            e = ParsingError(fpname)
                        e.append(lineno, repr(line))
        # if any parsing errors occurred, raise an exception
        if e:
            raise e

    def get_default_option(self, option):
        """ Get the given option from the default section """
        try:
            return self[DEFAULTSECT][option]
        except KeyError:
            raise NoOptionError(option, DEFAULTSECT)

    def get_opt(self, section, option):
        """ Return the option with leading and trailing quotes removed """
        optionstring = self[section][option].strip()
        if (optionstring[0] == '\'' and optionstring[-1] == '\'') or \
           (optionstring[0] == '\"' and optionstring[-1] == '\"'):
            optionstring = optionstring[1:-1]
        return optionstring

    def get_optlist(self, section, option, delim=','):
        """
        Return the option split into a list

        Required Arguments:
        section -- name of the section
        option -- name of the option

        Keyword Arguments:
        delim -- delimiter to use when splitting the option

        Returns:
        list -- option value split on 'delim' with whitespace trimmed

        """
        optionstring = self.get_opt( section, option )
        return [x.strip() for x in optionstring.split(delim)]

    def __add__(self, other):
        """
        Merge items from another ConfigManager

        Sections in 'other' will overwrite sections in 'self'.

        """
        other = other.copy()
        for key, value in list(other.items()):
            self[key] = value
        try:
           for key, value in list(other._categories.items()):
               self._categories[key] = value
        except AttributeError: pass
        return self

    def __iadd__(self, other):
        """ Merge items from another ConfigManager """
        return self.__add__(other)

    def __radd__(self, other):
        """
        Merge items from another ConfigManager

        Sections already in 'self' will not be overwritten.

        """
        other = other.copy()
        for key, value in list(other.items()):
            if key not in list(self.keys()):
                self[key] = value
        try:
           for key, values in list(other._categories.items()):
               if key not in list(self._categories.keys()):
                   self._categories[key] = value
        except AttributeError: pass
        return self

    def get_options_from_config(self):
        """
        Locate all short and long options

        Returns:
        tuple -- two element tuple contain a list of short option
           instances and a list of long option instances

        """
        short_prefixes, long_prefixes = type(self).get_prefixes()

        longopts = []
        shortopts = []
        for section in list(self.values()):
           for option in list(section.data.values()):
              for opt in option.getPossibleOptions():
                 opt = opt.replace('!','')

                 # See if the option is a long option
                 for prefix in long_prefixes:
                    if prefix is None:
                       pass
                    elif not prefix.strip():
                       pass
                    elif opt.startswith(prefix):
                       if option not in longopts:
                          longopts.append(option)
                       continue

                 # See if the option is a short option
                 for prefix in short_prefixes:
                    if prefix is None:
                       pass
                    elif not prefix.strip():
                       pass
                    elif opt.startswith(prefix):
                       if option not in shortopts:
                          shortopts.append(option)

        return shortopts, longopts

    def getopt(self, args=None, merge=1):
        """
        Parse the command line

        Keyword Arguments:
        args -- list of strings containing the command line.  If this is
           not given, sys.argv[1:] is used.
        merge -- boolean flag indicating whether parsed options should
           be merged into the configuration or not

        Returns:
        tuple -- two element tuple where the first element is a list of
           parsed options in the form '(option, value)' and the second
           element is a list of unparsed arguments

        """
        if args is None: args = sys.argv[1:]

        shortopts, longopts = self.get_options_from_config()

        short_prefixes, long_prefixes = type(self).get_prefixes()

        opts = []
        while args and args[0] not in short_prefixes:
            # If the argument is equal to one of the long prefixes,
            # throw it away and bail out.
            if args[0] in long_prefixes:
                args = args[1:]
                break

            # Parse long options
            if [x for x in long_prefixes if args[0].startswith(x)]:
               try:
                  opts, args = self.do_longs(opts,args[0],longopts,args[1:])
               except UnrecognizedArgument as e:
                  if self.strict: raise
                  opts, args = self.handle_unrecognized(e[1],opts,args,'long')
                  if merge: self.unrecognized.append(opts[-1])

            # Parse short options
            elif [x for x in short_prefixes if args[0].startswith(x)]:
               try:
                  opts, args = self.do_shorts(opts,args[0],shortopts,args[1:])
               except UnrecognizedArgument as e:
                  if self.strict: raise
                  opts, args = self.handle_unrecognized(e[1],opts,args,'short')
                  if merge: self.unrecognized.append(opts[-1])

            # No option found.  We're done.
            else:
                break

        # Merge command line options with configuration
        if merge:
           self.merge_options(opts)
           self.check_mandatory_options()

        return opts, args

    def check_mandatory_options(self):
        """
        Make sure that all mandatory options have been set

        """
        for section in self.values():
           for option in section.data.values():
              if not option.mandatory: continue
              if option.getValue() in [None,[]]:
                 names = ', '.join(option.getPossibleOptions())
                 if not names:
                    names = option.name
                 raise MandatoryOption('The %s option is mandatory, but was not given' % names, names)

    def handle_unrecognized(self, option, opts, args, type='long'):
        """
        Try to parse unrecognized options and their arguments

        Required Arguments:
        option -- the actual option parsed from the command-line
           which was not recognized
        opts -- tuples of already parsed options and their values
        args -- remaining unparsed command-line arguments

        Keyword Arguments:
        type -- flag indicating which type of argument to parse.
           This should be either "short" or "long".

        """
        if type == 'long':

           args.pop(0)

           # Explicitly specified value
           if option.find('=') + 1:
              option, value = option.split('=',1)
              opts.append((option, value))
              return opts, args

           # Implicitly specified value
           if self.has_following_argument(args):
              opts.append((option, args.pop(0)))
              return opts, args

           # No argument found
           opts.append((option, None))
           return opts, args

        elif type == 'short':

           short_prefixes, long_prefixes = self.get_prefixes()
           prefix = [x for x in short_prefixes if option.startswith(x)][0]

           start, end = args[0].split(option.replace(prefix,'',1),1)
           if end: args[0] = prefix + end
           else: args.pop(0)

           opts.append((option, ''))
           return opts, args

        raise ValueError('Expecting type of "short" or "long".')

    def merge_options(self, options):
        """ Merge options parsed from the command line into configuration """
        from plasTeX.ConfigManager.Generic import GenericOption
        from plasTeX.ConfigManager.Multi import MultiOption
        # Multi options that have been cleared by a command line option.
        # Lists will only be cleared on the first command line option, all
        # consecutive options will append.
        for option, value in options:

#           opts = self.getPossibleOptions()
#           negative = [x.replace('!','',1) for x in opts if x.startswith('!')]
#           positive = [x for x in opts if not x.startswith('!')]

            if isinstance(option, GenericOption):
                option.source = COMMANDLINE
                option.file = None
                if type(value) in [list, tuple]:
                    if not isinstance(option, MultiOption):
                        value = ' '.join(value)
                        option.occurrences += 1
                        option.setValue(value)
                    else:
                        if option.occurrences:
                            option.occurrences += 1
                            option += value
                        else:
                            option.occurrences += 1
                            option.setValue(value)
                else:
                    option.occurrences += 1
                    option.setValue(value)

    def get_prefixes(cls):
        """ Prepare option prefixes to make sure that they are always lists """
        long_prefixes = cls.long_prefix
        short_prefixes = cls.short_prefix
        if type(long_prefixes) not in [list, tuple]:
           long_prefixes = [long_prefixes]
        if type(short_prefixes) not in [list, tuple]:
           short_prefixes = [short_prefixes]
        return [x for x in short_prefixes if x],[x for x in long_prefixes if x]

    get_prefixes = classmethod(get_prefixes)

    def do_longs(self, opts, opt, longopts, args):
        """
        Parse a long option

        Required Arguments:
        opts -- list of parsed options
        opt -- string containing current option
        longopts -- list of long option instances
        args -- remaining argument list

        Returns:
        tuple -- two element tuple where the first argument is the
           'opts' list with the latest argument added and the second
           element is the 'args' list with the arguments from the
           current option removed

        """
        forcedarg = False
        if opt.find('=') + 1:
            forcedarg = True
            opt, arg = opt.split('=', 1)
            args.insert(0, arg)

        option = self.get_match(opt, longopts)
        option.actual = opt
        option.file = None

        # If we have an argument, but the option doesn't accept one, bail out.
        if forcedarg and not(option.acceptsArgument()):
            raise UnspecifiedArgument('option %s must not have an argument' \
                                       % opt, opt)

        elif forcedarg:
           optarg, args = option.getArgument(args, forcedarg=True)

        elif not(option.acceptsArgument()):
           pass

        # Parse the following arguments
        else:

           # See if we have a possible following argument
           if not type(self).has_following_argument(args):

              # No argument found, but we require one.
              if option.requiresArgument():
                 raise RequiresArgument('option %s requires argument' \
                                         % opt, opt)

           # Parse the argument
           optarg, args = option.getArgument(args)

        # Convert boolean options to proper value
        if not forcedarg and isinstance(option, BooleanOption):
           options = option.getPossibleOptions()
           negative = [x.replace('!','',1) for x in options
                                           if x.startswith('!')]
#          positive = [x for x in options if not x.startswith('!')]
           if opt in negative:
              optarg = 0
           else:
              optarg = 1

        opts.append((option, optarg))
        return opts, args

    def has_following_argument(cls, args):
        """
        Return boolean indicating the existence of a following argument

        Required Arguments:
        args -- command-line arguments to inspect for following argument.

        Returns:
        boolean -- true, if following argument exists

        """
        short_prefixes, long_prefixes = cls.get_prefixes()

        # No arguments at all
        if not(args):
           return 0

        # The next argument has an option prefix and it doesn't consist
        # entirely of a prefix.
        if [x for x in long_prefixes+short_prefixes if args[0].startswith(x)] \
           and args[0] not in long_prefixes+short_prefixes:
           return 0

        # All other cases fail.  This must be an argument.
        return 1

    has_following_argument = classmethod(has_following_argument)

    def get_match(self, opt, longopts):
        """
        Get possible matches for the given option

        Required Arguments:
        opt -- name of the current option
        longopts -- list of all long option instances

        Returns:
        instance -- an instance of the option which matches

        """
        possibilities = []
        for o in longopts:
           match = o.matches(opt)
           if match:
              possibilities.append((match, o))

        if not possibilities:
            raise UnrecognizedArgument('option %s not recognized' % opt, opt)

        # Is there an exact match?
        option = [x for x in possibilities if opt == x[0]]
        if option:
            return option[0][1]

        # No exact match, so better be unique.
        if len(possibilities) > 1:
            # XXX since possibilities contains all valid continuations,
            # might be nice to work them into the error msg
            raise NonUniquePrefix('option %s not a unique prefix' % opt, opt)

        assert len(possibilities) == 1

        return possibilities[0][1]

    def do_shorts(self, opts, optstring, shortopts, args):
        """
        Parse a short argument

        Required Arguments:
        opts -- list of parsed options
        optstring -- string containing current option
        shortopts -- list of short option instances
        args -- remaining argument list

        Returns:
        tuple -- two element tuple where the first argument is the
           'opts' list with the latest argument added and the second
           element is the 'args' list with the arguments from the
           current option removed

        """
        short_prefixes, long_prefixes = type(self).get_prefixes()

        optstring.strip()

        if not optstring:
           return [], args

        prefix = optstring[0]
        optstring = optstring[1:]

        while optstring != '':
            opt, optstring = optstring[0], optstring[1:]

            option = self.get_match(prefix+opt, shortopts)
            option.actual = prefix+opt
            option.file = None

            # See if we need to check for an argument
            if option.acceptsArgument():

                if optstring == '':

                   # Are there any following arguments?
                   if not type(self).has_following_argument(args):

                      # No argument found, but we require one.
                      if option.requiresArgument():
                         raise RequiresArgument('option %s requires argument' \
                                                 % opt, opt)

                   optarg, args = option.getArgument(args)

                else:
                   optarg, args = option.getArgument([optstring]+args)

                # No argument was found
                if args and args[0] == optstring:
                   optarg = None
                   optstring = args.pop(0)
                else:
                   optstring = ''

            else:
                optarg = None

            # Convert boolean options to proper value
            if optarg is None and isinstance(option, BooleanOption):
               options = option.getPossibleOptions()
               negative = [x.replace('!','',1) for x in options
                                               if x.startswith('!')]
#              positive = [x for x in options if not x.startswith('!')]
               if prefix+opt in negative:
                  optarg = 0
               else:
                  optarg = 1

            opts.append((option, optarg))

        return opts, args

    def usage_on(self, options):
        """
        Print full help for listed options and exit

        Required Arguments:
        options -- list of strings indicating which options to list
           help on (preceding '-' and '--' should be removed)

        """
        display = []
        for sectkey, section in list(self.items()):
           for optkey, option in list(section.items()):
              if option.long in options or option.short in options:
                 display.append((sectkey, optkey, option))

        display.reverse()

        err = sys.stderr.write

        while display:
           sectkey, optkey, opt = display.pop()
           err('Command Line: %s\n' % self._option_usage(opt))
           err('Configuration File: [%s] %s=\n' % (sectkey, optkey))

           current = opt.names()['current']
           if current != None: err('Current Value: %s\n\n' % current)
           else: err('\n')

           err('%s\n\n' % opt.description)
           if display: err('\n')

        sys.exit(1)

    def _option_usage(self, option):
        """ Return the option the way it can be typed at the command line """
        if option.options.strip():
           short_prefixes, long_prefixes = self.get_prefixes()
           prefixes = long_prefixes + short_prefixes
           options = re.sub(r'\s+', r' ', option.options.replace('!',''))
           options = options.split()
           options = [(len(x),x) for x in options
                       if [x for p in prefixes if x.startswith(p)]]
           options.sort()
           options = [x[1] for x in options]

           # Determine argument separator
           sep = ' '
           loptions = [x for x in options
                         if [x for p in long_prefixes if x.startswith(p)]]
           if loptions:
              sep = '='

           options = ', '.join(options)
           if option.acceptsArgument() and option.requiresArgument():
              return '%s%s%s' % (options, sep, option.synopsis)
           elif option.acceptsArgument():
              return '%s[%s%s]' % (options, sep, option.synopsis)
           else:
              return options
        return ''

    def usage(self, categories=None):
        """ Print descriptions of all command line options """
        categories = categories or []
        options = []
        for section in list(self.values()):
            for option in list(section.data.values()):
                if option.options:
                    options.append(option)
        options.sort()

        if not options: return ''

        name_len = 0
        summary_len = 0

        # Put options into categories
        categorized = {}
        for option in options:

            catname = self.get_category(option.category).strip()
            name = self._option_usage(option)

            summary = ''
            if option.summary:
               summary = option.summary % option.names()
               default = option.defaultValue()
               if default is not None:
                   summary += ' [%s]' % default

            if catname not in list(categorized.keys()):
               categorized[catname] = []
            categorized[catname].append((name,summary,option))

            if summary:
               name_len = max(name_len,len(name))
               summary_len = max(summary_len,len(summary))

        name_len = min(name_len, int(TERMINAL_WIDTH*MAX_NAME_WIDTH_RATIO))
        summary_len = TERMINAL_WIDTH - PREPAD - GUTTER - name_len

        # Build the output string
        s = ''
        if categories:
           categories = [self.get_category(x) for x in list(categories.keys())]
        else:
           categories = list(categorized.keys())
           categories.sort()
        for category in categories:
           options = categorized[category]
           if not category:
              if len(categories) > 1:
                 category = 'General Options'
              else:
                 category = 'Options'
           s += '\n%s:\n' % category
           for name, summary, option in options:
              length = len(name)
              summary = wrap(summary, summary_len)
              summary = ('\n%s' % (' '*(PREPAD+name_len+GUTTER))).join(summary)
              # Be lenient on the gutter if we are really close to
              # fitting in the allocated space
              format = '%s%s%s%s\n'
              colspace = max(GUTTER + name_len - length, GUTTER)
              if summary and ((name_len + GUTTER) > length):
                 colspace = (name_len + GUTTER) - length
              elif summary and length > name_len:
                 colspace = PREPAD + name_len + GUTTER
                 format = '%s%s\n%s%s\n'
              s += format % (' '*PREPAD, name, ' '*colspace, summary)
        return s


class CommandLineManager(OrderedDict):
   """ Command-Line Argument Manager """

   def __init__(self, data=None):
      OrderedDict.__init__(self, data or {})
      self._associations = {}

   def usage(self):
      s = ''
      for item in list(self.values()):
         if isinstance(item, ConfigManager):
            s += item.usage()
         else:
            break
      return s

   def requiresArgument(self):
      """ Return boolean indicating if config requires an argument """
      if not self: return 0
      for key in list(self.keys()):
         item = OrderedDict.__getitem__(self, key)
         if isinstance(item, GenericArgument):
             if item.mandatory:
                 return 1

   def getopt(self, args=None):
      if args == None:
         args = sys.argv[1:]
      else:
         args = args[:]

      if not self: return self

      for key in list(self.keys()):
         item = OrderedDict.__getitem__(self, key)
         association = self._associations.get(key, None)
         if isinstance(item, ConfigManager):
            if association:
               item.read(association)
            options, args = item.getopt(args)
         elif isinstance(item, GenericArgument):
            value, args = item.getArgument(args)
            item.setValue(value)
         else:
            raise ValueError("Unrecognized argument type.")

      if len(args):
         raise TooManyValues('Too many command-line arguments: %s' % ' '.join(args))

      return self

   def __setitem__(self, key, value):
      item = value
      if type(value) in [tuple, list]:
         value = list(value)
         item = value.pop(0)
         self._associations[key] = value
      assert isinstance(item, ConfigManager) or \
             isinstance(item, GenericArgument), \
             'Command-line parameters must be ConfigManagers or ' + \
             'subclasses of GenericArgument'
      if hasattr(item, 'name') and item.name is None:
         item.name = key
      OrderedDict.__setitem__(self, key, item)

   def __getitem__(self, key):
      if type(key) == slice:
         return self.__getslice__(key.start, key.stop)
      item = OrderedDict.__getitem__(self, key)
      if isinstance(item, ConfigManager):
         return item
      else:
         return item.getValue()



# These must be loaded last because they depend on variables
# assigned in this file.
from plasTeX.ConfigManager.Generic import GenericOption, GenericArgument
from plasTeX.ConfigManager.String import StringOption, StringArgument
from plasTeX.ConfigManager.Integer import IntegerOption, IntegerArgument
from plasTeX.ConfigManager.Float import FloatOption, FloatArgument
from plasTeX.ConfigManager.Multi import MultiOption, MultiArgument
from plasTeX.ConfigManager.Compound import CompoundOption, CompoundArgument
from plasTeX.ConfigManager.Boolean import BooleanOption, BooleanArgument
from plasTeX.ConfigManager.Files import OutputFileOption, InputFileOption
from plasTeX.ConfigManager.Directories import OutputDirectoryOption, InputDirectoryOption
from plasTeX.ConfigManager.Files import OutputFileArgument, InputFileArgument
from plasTeX.ConfigManager.Directories import OutputDirectoryArgument, InputDirectoryArgument
from plasTeX.ConfigManager.Counted import CountedOption, CountedArgument



#!/usr/bin/env python

import os, sys
import plasTeX.Imagers.gspdfpng as gspdfpng

gs = 'gs'
if sys.platform.startswith('win'):
   gs = 'gswin32c'

class GSDVIPNG(gspdfpng.GSPDFPNG):
    """ Imager that uses gs to convert dvi to png """
    compiler = 'latex'
    verification = '(%s --help && dvips --help)' % gs

    def executeConverter(self, output):
        open('images.dvi', 'w').write(output.read())
        rc = os.system('dvips -o images.ps images.dvi')
        if rc: return rc, None
        return gspdfpng.GSPDFPNG.executeConverter(self, open('images.ps'))

Imager = GSDVIPNG
